// interface graphique version trois pour le probleme a trois corps
// NB pour compilation javadoc : javadoc -author -link -version -d Documentation TroisCorps.java
import simu.*; 
import eqmvt.*;
import lagrange.*;
import vitessenulle.*;
import mn.*;
import analyse.*;
import java.lang.Math;

/**
* Algorithme de simulation pour SimuApplet.
* Le problme  trois corps plan restreint (deux masses principales et une masse test).
*
* <ul>
* <li><b>mu</b>                  <dd>paramtre qui est utilis pour caractriser les masses principales: soient m1 et m2 les deux masses en question, m1 est la plus grande des deux, alors, mu = m2 / (m1 + m2)</dd>
* <li><b>q1Initial</b>           <dd>coordonne 1 ddimensionne relative  la position de la sonde, valeur initiale</dd>    
* <li><b>q2Initial</b>           <dd>coordonne 2 ddimensionne relative  la position de la sonde, valeur initiale</dd>
* <li><b>q1dotInitial</b>        <dd>coordonne 1 ddimensionne relative  la vitesse de la sonde, valeur initiale</dd>
* <li><b>q2dotInitial</b>        <dd>coordonne 2 ddimensionne relative  la vitesse de la sonde, valeur initiale</dd>
* <li><b>p1Initial</b>           <dd>moment conjugu a q1 et q2 au moment initial</dd>
* <li><b>p2Initial</b>           <dd>moment conjugu a q1 et q2 au moment initial</dd>
* <li><b>deltaT</b>              <dd>pas d'intgration</dd>
* <li><b>dodo</b>                <dd>nombre qui sert a la temporisation, doit absolument tre un entier</dd>
* <li><b>periodeAffichage</b>    <dd>nombre d'itrations aprs lequel on affiche un point</dd>
*</ul>
*
* @author Aurelie GUILBERT (version du 17/01/2006)  
* @author Guillaume Boubin, Gatan Le Chat, Pierre INIZAN, Mathieu HAVEL (version du 19/01/2007)
*
* @version 19/01/2007
* @since Projet Trois Corps 2006
* 
* @see Integrateur
* @see PointFixe
* @see CourbeVitesse0
*
*/
public class TroisCorps extends SimuApplet {

   Integrateur monIntegrateur;    // pour le calcul de la trajectoire   
   InterfaceEqMvmt mouvement;     // pour le calcul de l'integrale de Jacobi
   String fEqMvmt = "EqMvmtPblmTroisCorps";  // nom lie au nom du fichier contenant l'equation du mouvement
   CourbeVitesse0 vitesseNulle;  //pour le calcul des courbes de vitesse nulle
   SectionPoincare sectionDePoincare;		 // pour le calcul de la section de Poincar

   double mu;                 //masse reduite du problme
   double[] mutab;            // tableau de mu pour le calcul

   double q1Initial, q2Initial;          // position dans le repere tournant, initiale, suivant l'axe des masses
   double q1dotInitial, q2dotInitial;  // vitesses initiales 
   double p1Initial, p2Initial;        // moments conjugues a q1 et q2 au moment initial 

   double deltaT;			// pas d'integration
   int dodo;				// temporisation
   int periodeAffichage=1;	// nombre d'itrations aprs lequel on affiche un point

   double[] voyager;		// tableau qui va ensuite servir pour le calcul: contient q1 q2 q1dot et q2dot
   
   double[] q;				//tableau servant au calcul des courbes de vitesses nulles: contient q1 et q2
   double[] qq1;			//tableau servant au calcul des courbes de vitesses nulles: contient q11 et q21
   double[] qq2;			//tableau servant au calcul des courbes de vitesses nulles: contient q12 et q22
   double[] qI;				//position initiales de la continuation des courbes de vitesses nulles
   double[] qI1;			//position initiales de la continuation des courbes de vitesses nulles en q11 et q21
   double[] qI2;			//position initiales de la continuation des courbes de vitesses nulles en q12 et q22
   double[] stop;			//tableau permettant de stopper la continuation
   double[] origin;			// point d'origine a partir duquel on calcul qI
   double[] origin1;		// point d'origine a partir duquel on calcul qI1
   double[] origin2;		// point d'origine a partir duquel on calcul qI2
   int trajV0;				// permet de tracer la trajectoire aprs les courbes de vitesses nulles

   String choix="traj";		// String determinant le trac des courbes : traj => trac de la trajectoire, vite => trac des courbes de vitesses nulles
   							// sect => trac de la section de Poincar
   
   int appel;				// entier qui sert a construire la graphe en comptant les appels
   
   // Section de Poincar
   String sectionPoincare;	// Section de Poincar "on" ou "off"
   boolean rester;
   int appelP;				// compteur d'appel  RK4symp sans passer couper la surface de section.
   int nbPtsTrajP;			// entier qui dtermine combien de points on souhaite pour 1 trajectoire de Poincar (Section de Poincar)
   int ptsTrajP;			// compte le nombre de points tracs sur une trajectoire de Poincar
   double pasp10;		// pas d'incrmentation de p1Initial pour la section de Poincar
   double pasq10;		// pas d'incrmentation de q1Initial pour la section de Poincar
   double surfaceSection;	// surface de Section que l'on regarde (pour l'instant toujours q2 = 0)
   double p20a,p20b;		// p2Initial ncessaires pour rester sur la varit d'nergie intiale.
   int solutionP;           // Y-a-t il une solution de p2Initial pour rester sur la varit ?
   int solutionPa;          // solutions des nouvelles positions avec p20a ; =0 si pas de solution, =1 sinon
   int solutionPb;          // solutions des nouvelles positions avec p20b ; =0 si pas de solution, =1 sinon
   double[] voyagerAll;		// contient les Q1,Q2=0,P1,P2 pour chaque P20 possible + le nombre d'appel + solution ? (=0 ou 1)
   double p10,q10;			// valeurs courantes de p10 et q10 pour la trajectoire de Poincare calcule
   double p10max,q10max;	// valeurs maximum de p10 et q10
   double p10min,q10min;	// valeurs minimum de p10 et q10
   int compteurq10;			// compte le nombre de fois o q10max a t atteint.
   double jacobiP;			// valeur de l'intgrale de Jacobi pour la section de Poincar trace.
   double j0;				// valeur de l'intgrale de Jacobi pour la section de Poincar trace (paramIn)

   static  double jacobiInitial; // valeur de l'integrale de jacobi pour les conditions initiales

   // Courbes de vitesse nulle
   String courbeTrajectoire;	// est ce qu'on veut tracer les courbes de vitesse nulle et la trajectoire ?
   String courbe;				// est ce qu'on veut tracer les courbes de vitesse nulle ?
   static double[] vitesse;

   // Intgrateur
   String[] choixInteg;			//stocke les differents integrateurs
   String typeInteg;			//choix de l'integrateur stockes precedemment
   double relaxation;			// relaxation (coeff) appliqu  la correction  chaque itration pour RK4 symplectique

   // declaration des graphiques
   Affichage graphe = new Affichage();  // plot live dans repere tournant
   //Affichage fixe = new Affichage();    // plot live dans le repere fixe
   Affichage multi = new Affichage();   // multiplot avec Jacobi
   Affichage m1 = new Affichage();      // sous partie 1 du multiplot
   Affichage m2 = new Affichage();      // sous partie 2 du multiplot
   Affichage toufix = new Affichage();  // multiplot avec repere fixe
   Affichage tf1 = new Affichage();     // sous partie 1
   Affichage tf2 = new Affichage();     // sous partie 2
   Affichage courbeVitesse = new Affichage();     // courbe de vitesse nulle
   Affichage sectionTraj = new Affichage(); // section de Poincar + traj repre tournant
   Affichage section1 = new Affichage(); // Section seule
   Affichage section2 = new Affichage(); // traj repre tournant seule
   
   public TroisCorps() {

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//************************************************************//
//                     PARAMETRES D'ENTREE                    //
//************************************************************//
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// ********* ---concernant les deux masses princiales...--- *********
// definition typique d'un parametre d'entree pour simuapplet

  ParamIn mu = new ParamIn();
  mu.type = "nombre";
  mu.label = "mu";
  mu.titre = "Mu";
  mu.acquisition = "champ";
  mu.defaut = "0.5";
  addParamIn(mu);

// ********* --- integration--- *********

  // pas d'integration
  ParamIn pas = new ParamIn();
  pas.type = "nombre";
  pas.label = "pasintegration";
  pas.titre = "Pas d'integration";
  pas.acquisition = "champ";
  pas.defaut = "0.001";
  addParamIn(pas);

  // temporisation
  ParamIn dodo = new ParamIn();
  dodo.type = "nombre";
  dodo.label = "ralentissement";
  dodo.titre = "Ralentissement";
  dodo.acquisition = "champ";
  dodo.defaut = "10";
  addParamIn(dodo);

  // Periode d'affichage des points
  ParamIn periodeAffichage = new ParamIn();
  periodeAffichage.type = "nombre";
  periodeAffichage.label = "periodeAff";
  periodeAffichage.titre = "Priode d'aff.|Nb. pts/traject. Poincar";
  periodeAffichage.acquisition = "champ";
  periodeAffichage.defaut = "1";
  addParamIn(periodeAffichage);

  // Choix de l'intgrateur
  choixInteg = new String[2];
  choixInteg[0] = "RK4  pas variable";
  choixInteg[1] = "RK4 symplectique  pas fixe";
  ParamIn integ = new ParamIn();
  integ.type = "string";
  integ.label = "integrateur";
  integ.titre = "Intgrateur utilis";
  integ.acquisition = "choix";
  integ.choix = choixInteg;
  integ.defaut = choixInteg[0];
  addParamIn(integ);

  // ************ Section de Poincar *************
  
  /*// Nombre d'itrations par trajectoire de Poincar
  ParamIn nbP = new ParamIn();
  nbP.type = "nombre";
  nbP.label = "iterP";
  nbP.titre = "Nb. de points / traj. de Poincar";
  nbP.acquisition = "champ";
  nbP.defaut = "100";
  addParamIn(nbP);*/
  
  /*
  // p10min
  ParamIn minp10 = new ParamIn();
  minp10.type = "nombre";
  minp10.label = "MinP10";
  minp10.titre = "Section Poincar : P10 min";
  minp10.acquisition = "champ";
  minp10.defaut = "-0.005";
  addParamIn(minp10);
  
  // p10max
  ParamIn maxp10 = new ParamIn();
  maxp10.type = "nombre";
  maxp10.label = "MaxP10";
  maxp10.titre = "Section Poincar : P10 max";
  maxp10.acquisition = "champ";
  maxp10.defaut = "0.005";
  addParamIn(maxp10);
  
  // q10min
  ParamIn minq10 = new ParamIn();
  minq10.type = "nombre";
  minq10.label = "MinQ10";
  minq10.titre = "Section Poincar : Q10 min";
  minq10.acquisition = "champ";
  minq10.defaut = "0.1";
  addParamIn(minq10);
  
  // q10max
  ParamIn maxq10 = new ParamIn();
  maxq10.type = "nombre";
  maxq10.label = "MaxQ10";
  maxq10.titre = "Section Poincar : Q10 max";
  maxq10.acquisition = "champ";
  maxq10.defaut = "0.6";
  addParamIn(maxq10);
  
  // pas p10
  ParamIn deltap10 = new ParamIn();
  deltap10.type = "nombre";
  deltap10.label = "DP10";
  deltap10.titre = "Section Poincar : delta_P10";
  deltap10.acquisition = "champ";
  deltap10.defaut = "0.0005";
  addParamIn(deltap10);
  
  // pas q10
  ParamIn deltaq10 = new ParamIn();
  deltaq10.type = "nombre";
  deltaq10.label = "DP10";
  deltaq10.titre = "Section Poincar : delta_Q10";
  deltaq10.acquisition = "champ";
  deltaq10.defaut = "0.08";
  addParamIn(deltaq10);*/
  
  // Intgrale de Jacobi (valeur prise en compte uniquement pour les sections de Poincar)
  ParamIn j0 = new ParamIn();
  j0.type = "nombre";
  j0.label = "J0";
  j0.titre = "Section Poincar : Jacobi fix ";
  j0.acquisition = "champ";
  j0.defaut = "4.0";
  addParamIn(j0);
  
// ********* ---concernant la sonde...--- *********

  // position initiale de la sonde, elle va etre de fait donnee dans le centre de masse
  ParamIn posx = new ParamIn();
  posx.type = "nombre";
  posx.label = "q1";
  posx.titre = "q1";
  posx.acquisition = "champ";
  posx.defaut = "0.0";
  addParamIn(posx);

  ParamIn posy = new ParamIn();
  posy.type = "nombre";
  posy.label = "q2";
  posy.titre = "q2";
  posy.acquisition = "champ";
  posy.defaut = "0.5";
  addParamIn(posy);

  // vitesse initiale de la sonde
  ParamIn vx = new ParamIn();
  vx.type = "nombre";
  vx.label = "q1point";
  vx.titre = "q1point";
  vx.acquisition = "champ";
  vx.defaut = "0";
  addParamIn(vx);

  ParamIn vy = new ParamIn();
  vy.type = "nombre";
  vy.label = "q2point";
  vy.titre = "q2point";
  vy.acquisition = "champ";
  vy.defaut = "0";
  addParamIn(vy);

// ********* ---ajustement automatique de la fenetre graphique--- *********

  ParamIn ajust = new ParamIn();
  ajust.type = "string";
  ajust.label = "ajustement";
  ajust.titre = "Ajustement automatique de la fentre graphique ?";
  ajust.acquisition = "case"; 
  ajust.defaut = "off";
  addParamIn(ajust);

// ********* ---voir les courbes de vitesse nulle ou pas ?--- *********

  ParamIn courbe = new ParamIn();
  courbe.type = "string";
  courbe.label = "vitessenulle";
  courbe.titre = "Voir juste les courbes de vitesse nulle ?";
  courbe.acquisition = "case"; 
  courbe.defaut = "off";
  addParamIn(courbe);

// ********* ---voir les courbes de vitesse nulle et la trajectoire ?--- *******

  ParamIn courbeTrajectoire = new ParamIn();
  courbeTrajectoire.type = "string";
  courbeTrajectoire.label = "vitessenulleEtTrajectoire";
  courbeTrajectoire.titre = "Voir les courbes de vitesse nulle et la trajectoire ?";
  courbeTrajectoire.acquisition = "case"; 
  courbeTrajectoire.defaut = "off";
  addParamIn(courbeTrajectoire);
  
//********* ---voir la section de Poincar q2=0 ?--- *******

  ParamIn sectionPoinc = new ParamIn();
  sectionPoinc.type = "string";
  sectionPoinc.label = "sectionP";
  sectionPoinc.titre = "Voir la section de Poincar ?";
  sectionPoinc.acquisition = "case"; 
  sectionPoinc.defaut = "off";
  addParamIn(sectionPoinc);
  
//==========================================
//     ENSEMBLES DE VALEURS PREDEFINIES
//==========================================
// ********* ---ajout d'une liste de valeur predefinies pour simplifier l'utilisation de cas typiques--- *********

  // trajectoire de henon
  ListeValeurs henon_rk4_pv = new ListeValeurs();
  henon_rk4_pv.ajouterDouble( "mu", 0.5 );
  henon_rk4_pv.ajouterDouble("q1",  -0.267944);
  henon_rk4_pv.ajouterDouble("q2", 0);
  henon_rk4_pv.ajouterDouble("q1point", 0.0 );
  henon_rk4_pv.ajouterDouble("q2point", 1.82134);
  henon_rk4_pv.ajouterDouble("pasintegration", 0.05);
  henon_rk4_pv.ajouterEntier("ralentissement", 100);
  henon_rk4_pv.ajouterEntier("periodeAff", 1);
  //henon_rk4_pv.ajouterEntier("nbP", 100);
  henon_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  henon_rk4_pv.ajouterDouble("J0", 2.36599752597);
  addEnsemble("Trajectoire de Hnon RK4_PV", henon_rk4_pv);

  // trajectoire de henon du RK4 symplectique
  ListeValeurs henon_rk4_symp = new ListeValeurs();
  henon_rk4_symp.ajouterDouble( "mu", 0.5 );
  henon_rk4_symp.ajouterDouble("q1",  -0.267944);
  henon_rk4_symp.ajouterDouble("q2", 0);
  henon_rk4_symp.ajouterDouble("q1point", 0.0 );
  henon_rk4_symp.ajouterDouble("q2point", 1.82134);
  henon_rk4_symp.ajouterDouble("pasintegration", 0.005);
  henon_rk4_symp.ajouterEntier("ralentissement", 100);
  henon_rk4_symp.ajouterEntier("periodeAff", 10);
  //henon_rk4_symp.ajouterEntier("nbP", 100);
  henon_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  henon_rk4_symp.ajouterDouble("J0", 2.36599752597);
  addEnsemble("Trajectoire de Hnon RK4_SYMP", henon_rk4_symp);
  
  // SECTION trajectoire de henon du RK4 symplectique
  ListeValeurs henon_sect = new ListeValeurs();
  henon_sect.ajouterDouble( "mu", 0.5 );
  henon_sect.ajouterDouble("q1",  -0.267944);
  henon_sect.ajouterDouble("q2", 0);
  henon_sect.ajouterDouble("q1point", 0.0 );
  henon_sect.ajouterDouble("q2point", 1.82134);
  henon_sect.ajouterDouble("pasintegration", 0.005);
  henon_sect.ajouterEntier("ralentissement", 100);
  henon_sect.ajouterEntier("periodeAff", 400);
  henon_sect.ajouterString("integrateur", choixInteg[1]);
  henon_sect.ajouterDouble("J0", 2.36599752597);
  addEnsemble("Trajectoire de Hnon SECTION", henon_sect);

  // haltere
  ListeValeurs jolie_rk4_pv = new ListeValeurs();
  jolie_rk4_pv.ajouterDouble( "mu", 0.02);
  jolie_rk4_pv.ajouterDouble("q1", 0.48);
  jolie_rk4_pv.ajouterDouble("q2", 0.966025);
  jolie_rk4_pv.ajouterDouble("q1point", 0.5);
  jolie_rk4_pv.ajouterDouble("q2point", -0.25);
  jolie_rk4_pv.ajouterDouble("pasintegration", 0.05);
  jolie_rk4_pv.ajouterEntier("ralentissement", 0);
  jolie_rk4_pv.ajouterEntier("periodeAff", 1);
  //jolie_rk4_pv.ajouterEntier("nbP", 100);
  jolie_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  jolie_rk4_pv.ajouterDouble("J0", 2.6897587874);
  addEnsemble("Haltre RK4_PV", jolie_rk4_pv );

  // haltere du RK4 symplectique
  ListeValeurs jolie_rk4_symp = new ListeValeurs();
  jolie_rk4_symp.ajouterDouble( "mu", 0.02);
  jolie_rk4_symp.ajouterDouble("q1", 0.48);
  jolie_rk4_symp.ajouterDouble("q2", 0.966025);
  jolie_rk4_symp.ajouterDouble("q1point", 0.5);
  jolie_rk4_symp.ajouterDouble("q2point", -0.25);
  jolie_rk4_symp.ajouterDouble("pasintegration", 0.005);
  jolie_rk4_symp.ajouterEntier("ralentissement", 0);
  jolie_rk4_symp.ajouterEntier("periodeAff", 10);
  //jolie_rk4_symp.ajouterEntier("nbP", 100);
  jolie_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  jolie_rk4_symp.ajouterDouble("J0", 2.6897587874);
  addEnsemble("Haltre RK4_SYMP", jolie_rk4_symp );
  
  // SECTION haltere du RK4 symplectique
  ListeValeurs jolie_sect = new ListeValeurs();
  jolie_sect.ajouterDouble( "mu", 0.02);
  jolie_sect.ajouterDouble("q1", 0.48);
  jolie_sect.ajouterDouble("q2", 0.966025);
  jolie_sect.ajouterDouble("q1point", 0.5);
  jolie_sect.ajouterDouble("q2point", -0.25);
  jolie_sect.ajouterDouble("pasintegration", 0.005);
  jolie_sect.ajouterEntier("ralentissement", 0);
  jolie_sect.ajouterEntier("periodeAff", 400);
  jolie_sect.ajouterString("integrateur", choixInteg[1]);
  jolie_sect.ajouterDouble("J0", 2.6897587874);
  addEnsemble("Haltre SECTION", jolie_sect );

  // trajectoire pres de L4
  ListeValeurs pres_rk4_pv = new ListeValeurs();
  pres_rk4_pv.ajouterDouble("mu", 0.02);
  pres_rk4_pv.ajouterDouble("q1", 0.5);
  pres_rk4_pv.ajouterDouble("q2", 0.866025);
  pres_rk4_pv.ajouterDouble("q1point", 0.0);
  pres_rk4_pv.ajouterDouble("q2opint", -0.02);
  pres_rk4_pv.ajouterDouble("pasintegration", 0.5);
  pres_rk4_pv.ajouterEntier("ralentissement", 0);
  pres_rk4_pv.ajouterEntier("periodeAff", 1);
  //pres_rk4_pv.ajouterEntier("nbP", 100);
  pres_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  pres_rk4_pv.ajouterDouble("J0", 2.9182066069);
  addEnsemble("Trajectoire autour de L4 RK4_PV", pres_rk4_pv);

  // trajectoire pres de L4 du RK4 symplectique
  ListeValeurs pres_rk4_symp = new ListeValeurs();
  pres_rk4_symp.ajouterDouble("mu", 0.02);
  pres_rk4_symp.ajouterDouble("q1", 0.5);
  pres_rk4_symp.ajouterDouble("q2", 0.866025);
  pres_rk4_symp.ajouterDouble("q1point", 0.0);
  pres_rk4_symp.ajouterDouble("q2opint", -0.02);
  pres_rk4_symp.ajouterDouble("pasintegration", 0.05);
  pres_rk4_symp.ajouterEntier("ralentissement", 0);
  pres_rk4_symp.ajouterEntier("periodeAff", 10);
  //pres_rk4_symp.ajouterEntier("nbP", 100);
  pres_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  pres_rk4_symp.ajouterDouble("J0", 2.9182066069);
  addEnsemble("Trajectoire autour de L4 RK4_SYMP", pres_rk4_symp);
  
  // SECTION trajectoire pres de L4 du RK4 symplectique
  ListeValeurs pres_sect = new ListeValeurs();
  pres_sect.ajouterDouble("mu", 0.02);
  pres_sect.ajouterDouble("q1", 0.5);
  pres_sect.ajouterDouble("q2", 0.866025);
  pres_sect.ajouterDouble("q1point", 0.0);
  pres_sect.ajouterDouble("q2opint", -0.02);
  pres_sect.ajouterDouble("pasintegration", 0.05);
  pres_sect.ajouterEntier("ralentissement", 0);
  pres_sect.ajouterEntier("periodeAff", 400);
  pres_sect.ajouterString("integrateur", choixInteg[1]);
  pres_sect.ajouterDouble("J0", 2.9182066069);
  addEnsemble("Trajectoire autour de L4 SECTION", pres_sect);

  // banane
  ListeValeurs banane_rk4_pv = new ListeValeurs();
  banane_rk4_pv.ajouterDouble( "mu", 0.0009547861);
  banane_rk4_pv.ajouterDouble("q1", 0.525000000);
  banane_rk4_pv.ajouterDouble("q2", -0.909326674);
  banane_rk4_pv.ajouterDouble("q1point", -0.064718610);
  banane_rk4_pv.ajouterDouble("q2point", -0.044371494);
  banane_rk4_pv.ajouterDouble("pasintegration", 0.5);
  banane_rk4_pv.ajouterEntier("ralentissement", 0);
  banane_rk4_pv.ajouterEntier("periodeAff", 1);
  //banane_rk4_pv.ajouterEntier("nbP", 100);
  banane_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  banane_rk4_pv.ajouterDouble("J0", 3.000282687137);
  addEnsemble("Trajectoire autour de L5 RK4_PV", banane_rk4_pv);

  // banane du RK4 symplectique
  ListeValeurs banane_rk4_symp = new ListeValeurs();
  banane_rk4_symp.ajouterDouble( "mu", 0.0009547861);
  banane_rk4_symp.ajouterDouble("q1", 0.525000000);
  banane_rk4_symp.ajouterDouble("q2", -0.909326674);
  banane_rk4_symp.ajouterDouble("q1point", -0.064718610);
  banane_rk4_symp.ajouterDouble("q2point", -0.044371494);
  banane_rk4_symp.ajouterDouble("pasintegration", 0.05);
  banane_rk4_symp.ajouterEntier("ralentissement", 0);
  banane_rk4_symp.ajouterEntier("periodeAff", 10);
  //banane_rk4_symp.ajouterEntier("nbP", 100);
  banane_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  banane_rk4_symp.ajouterDouble("J0", 3.000282687137);
  addEnsemble("Trajectoire autour de L5 RK4_SYMP", banane_rk4_symp);

  // SECTION banane du RK4 symplectique
  ListeValeurs banane_sect = new ListeValeurs();
  banane_sect.ajouterDouble( "mu", 0.0009547861);
  banane_sect.ajouterDouble("q1", 0.525000000);
  banane_sect.ajouterDouble("q2", -0.909326674);
  banane_sect.ajouterDouble("q1point", -0.064718610);
  banane_sect.ajouterDouble("q2point", -0.044371494);
  banane_sect.ajouterDouble("pasintegration", 0.05);
  banane_sect.ajouterEntier("ralentissement", 0);
  banane_sect.ajouterEntier("periodeAff", 400);
  banane_sect.ajouterString("integrateur", choixInteg[1]);
  banane_sect.ajouterDouble("J0", 3.000282687137);
  addEnsemble("Trajectoire autour de L5 SECTION", banane_sect);
  
  // fer a cheval 4
  ListeValeurs horse4_rk4_pv = new ListeValeurs();
  horse4_rk4_pv.ajouterDouble( "mu", 0.0009547861);
  horse4_rk4_pv.ajouterDouble("q1", 0.520000000);
  horse4_rk4_pv.ajouterDouble("q2", 0.900666420);
  horse4_rk4_pv.ajouterDouble("q1point", 0.051579626);
  horse4_rk4_pv.ajouterDouble("q2point", -0.029668175);
  horse4_rk4_pv.ajouterDouble("pasintegration", 1.);
  horse4_rk4_pv.ajouterEntier("ralentissement", 0);
  horse4_rk4_pv.ajouterEntier("periodeAff", 1);
  //horse4_rk4_pv.ajouterEntier("nbP", 100);
  horse4_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  horse4_rk4_pv.ajouterDouble("J0", 3.00028989906);
  addEnsemble("Fer  cheval 1 RK4_PV", horse4_rk4_pv);

  // fer a cheval 4 du RK4 symplectique
  ListeValeurs horse4_rk4_symp = new ListeValeurs();
  horse4_rk4_symp.ajouterDouble( "mu", 0.0009547861);
  horse4_rk4_symp.ajouterDouble("q1", 0.520000000);
  horse4_rk4_symp.ajouterDouble("q2", 0.900666420);
  horse4_rk4_symp.ajouterDouble("q1point", 0.051579626);
  horse4_rk4_symp.ajouterDouble("q2point", -0.029668175);
  horse4_rk4_symp.ajouterDouble("pasintegration", 0.1);
  horse4_rk4_symp.ajouterEntier("ralentissement", 0);
  horse4_rk4_symp.ajouterEntier("periodeAff", 10);
  //horse4_rk4_symp.ajouterEntier("nbP", 100);
  horse4_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  horse4_rk4_symp.ajouterDouble("J0", 3.00028989906);
  addEnsemble("Fer  cheval 1 RK4_SYMP", horse4_rk4_symp);
  
  // SECTION fer a cheval 4 du RK4 symplectique
  ListeValeurs horse4_sect = new ListeValeurs();
  horse4_sect.ajouterDouble( "mu", 0.0009547861);
  horse4_sect.ajouterDouble("q1", 0.520000000);
  horse4_sect.ajouterDouble("q2", 0.900666420);
  horse4_sect.ajouterDouble("q1point", 0.051579626);
  horse4_sect.ajouterDouble("q2point", -0.029668175);
  horse4_sect.ajouterDouble("pasintegration", 0.1);
  horse4_sect.ajouterEntier("ralentissement", 0);
  horse4_sect.ajouterEntier("periodeAff", 400);
  horse4_sect.ajouterString("integrateur", choixInteg[1]);
  horse4_sect.ajouterDouble("J0", 3.00028989906);
  addEnsemble("Fer  cheval 1 SECTION", horse4_sect);

  // fer a cheval 6
  ListeValeurs horse6_rk4_pv = new ListeValeurs();
  horse6_rk4_pv.ajouterDouble( "mu", 0.0009547861);
  horse6_rk4_pv.ajouterDouble("q1", 0.53);
  horse6_rk4_pv.ajouterDouble("q2", 0.917986928);
  horse6_rk4_pv.ajouterDouble("q1point", 0.077336855);
  horse6_rk4_pv.ajouterDouble("q2point", -0.044371494);
  horse6_rk4_pv.ajouterDouble("pasintegration", 0.3);
  horse6_rk4_pv.ajouterEntier("ralentissement", 10);
  horse6_rk4_pv.ajouterEntier("periodeAff", 1);
  //horse6_rk4_pv.ajouterEntier("nbP", 100);
  horse6_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  horse6_rk4_pv.ajouterDouble("J0", 3.001644395376);
  addEnsemble("Fer a cheval 2 RK4_PV", horse6_rk4_pv);

  // fer a cheval 6 du RK4 symplectique
  ListeValeurs horse6_rk4_symp = new ListeValeurs();
  horse6_rk4_symp.ajouterDouble( "mu", 0.0009547861);
  horse6_rk4_symp.ajouterDouble("q1", 0.53);
  horse6_rk4_symp.ajouterDouble("q2", 0.917986928);
  horse6_rk4_symp.ajouterDouble("q1point", 0.077336855);
  horse6_rk4_symp.ajouterDouble("q2point", -0.044371494);
  horse6_rk4_symp.ajouterDouble("pasintegration", 0.03);
  horse6_rk4_symp.ajouterEntier("ralentissement", 10);
  horse6_rk4_symp.ajouterEntier("periodeAff", 10);
  //horse6_rk4_symp.ajouterEntier("nbP", 100);
  horse6_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  horse6_rk4_symp.ajouterDouble("J0", 3.001644395376);
  addEnsemble("Fer a cheval 2 RK4_SYMP", horse6_rk4_symp);
  
  // SECTION fer a cheval 6 du RK4 symplectique
  ListeValeurs horse6_sect = new ListeValeurs();
  horse6_sect.ajouterDouble( "mu", 0.0009547861);
  horse6_sect.ajouterDouble("q1", 0.53);
  horse6_sect.ajouterDouble("q2", 0.917986928);
  horse6_sect.ajouterDouble("q1point", 0.077336855);
  horse6_sect.ajouterDouble("q2point", -0.044371494);
  horse6_sect.ajouterDouble("pasintegration", 0.03);
  horse6_sect.ajouterEntier("ralentissement", 10);
  horse6_sect.ajouterEntier("periodeAff", 400);
  horse6_sect.ajouterString("integrateur", choixInteg[1]);
  horse6_sect.ajouterDouble("J0", 3.001644395376);
  addEnsemble("Fer a cheval 2 SECTION", horse6_sect);

  // crash sur Jupiter 1 (RK4 symplectique)
  ListeValeurs crash1 = new ListeValeurs();
  crash1.ajouterDouble("mu", 0.0009547861);
  crash1.ajouterDouble("q1", 0.5);
  crash1.ajouterDouble("q2", 0.86);
  crash1.ajouterDouble("q1point", 0.0);
  crash1.ajouterDouble("q2point", -0.1);
  crash1.ajouterDouble("pasintegration", 0.03);
  crash1.ajouterEntier("ralentissement", 0);
  crash1.ajouterEntier("periodeAff", 1);
  //crash1.ajouterEntier("nbP", 100);
  crash1.ajouterString("integrateur", choixInteg[1]);
  crash1.ajouterDouble("J0", 2.989113581343474);
  addEnsemble("Crash sur Jupiter RK4_SYMP", crash1);

  // tube 2
  ListeValeurs tub2_rk4_pv = new ListeValeurs();
  tub2_rk4_pv.ajouterDouble( "mu", 0.0009547861);
  tub2_rk4_pv.ajouterDouble("q1", 0.495);
  tub2_rk4_pv.ajouterDouble("q2", 0.874685658);
  tub2_rk4_pv.ajouterDouble("q1point", 0.012987562);
  tub2_rk4_pv.ajouterDouble("q2point", 0.007488536);
  tub2_rk4_pv.ajouterDouble("pasintegration", 0.5);
  tub2_rk4_pv.ajouterEntier("ralentissement", 0);
  tub2_rk4_pv.ajouterEntier("periodeAff", 1);
  //tub2_rk4_pv.ajouterEntier("nbP", 100);
  tub2_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  tub2_rk4_pv.ajouterDouble("J0", 2.99891222087);
  addEnsemble("Tube autour de L4 RK4_PV", tub2_rk4_pv);

  // tube 2 du RK4 symplectique
  ListeValeurs tub2_rk4_symp = new ListeValeurs();
  tub2_rk4_symp.ajouterDouble( "mu", 0.0009547861);
  tub2_rk4_symp.ajouterDouble("q1", 0.495);
  tub2_rk4_symp.ajouterDouble("q2", 0.874685658);
  tub2_rk4_symp.ajouterDouble("q1point", 0.012987562);
  tub2_rk4_symp.ajouterDouble("q2point", 0.007488536);
  tub2_rk4_symp.ajouterDouble("pasintegration", 0.05);
  tub2_rk4_symp.ajouterEntier("ralentissement", 0);
  tub2_rk4_symp.ajouterEntier("periodeAff", 10);
  //tub2_rk4_symp.ajouterEntier("nbP", 100);
  tub2_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  tub2_rk4_symp.ajouterDouble("J0", 2.99891222087);
  addEnsemble("Tube autour de L4 RK4_SYMP", tub2_rk4_symp);
  
  // SECTION tube 2 du RK4 symplectique
  ListeValeurs tub2_sect = new ListeValeurs();
  tub2_sect.ajouterDouble( "mu", 0.0009547861);
  tub2_sect.ajouterDouble("q1", 0.495);
  tub2_sect.ajouterDouble("q2", 0.874685658);
  tub2_sect.ajouterDouble("q1point", 0.012987562);
  tub2_sect.ajouterDouble("q2point", 0.007488536);
  tub2_sect.ajouterDouble("pasintegration", 0.05);
  tub2_sect.ajouterEntier("ralentissement", 0);
  tub2_sect.ajouterEntier("periodeAff", 400);
  tub2_sect.ajouterString("integrateur", choixInteg[1]);
  tub2_sect.ajouterDouble("J0", 2.99891222087);
  addEnsemble("Tube autour de L4 SECTION", tub2_sect);

  // helio
  ListeValeurs helio_rk4_pv = new ListeValeurs();
  helio_rk4_pv.ajouterDouble( "mu", 0.0009547861);
  helio_rk4_pv.ajouterDouble("q1", 0.530000000);
  helio_rk4_pv.ajouterDouble("q2", 0.082013072);
  helio_rk4_pv.ajouterDouble("q1point", 0.077336855);
  helio_rk4_pv.ajouterDouble("q2point", -0.044371494);
  helio_rk4_pv.ajouterDouble("pasintegration", 0.01);
  helio_rk4_pv.ajouterEntier("ralentissement", 0);
  helio_rk4_pv.ajouterEntier("periodeAff", 1);
  //helio_rk4_pv.ajouterEntier("nbP", 100);
  helio_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  helio_rk4_pv.ajouterDouble("J0", 4.002783939123135);
  addEnsemble("Trajectoire heliocentrique RK4_PV", helio_rk4_pv);

  // helio du RK4 symplectique
  ListeValeurs helio_rk4_symp = new ListeValeurs();
  helio_rk4_symp.ajouterDouble( "mu", 0.0009547861);
  helio_rk4_symp.ajouterDouble("q1", 0.530000000);
  helio_rk4_symp.ajouterDouble("q2", 0.082013072);
  helio_rk4_symp.ajouterDouble("q1point", 0.077336855);
  helio_rk4_symp.ajouterDouble("q2point", -0.044371494);
  helio_rk4_symp.ajouterDouble("pasintegration", 0.001);
  helio_rk4_symp.ajouterEntier("ralentissement", 0);
  helio_rk4_symp.ajouterEntier("periodeAff", 10);
  //helio_rk4_symp.ajouterEntier("nbP", 100);
  helio_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  helio_rk4_symp.ajouterDouble("J0", 4.002783939123135);
  addEnsemble("Trajectoire heliocentrique RK4_SYMP", helio_rk4_symp);
  
  // SECTION helio du RK4 symplectique
  ListeValeurs helio_sect = new ListeValeurs();
  helio_sect.ajouterDouble( "mu", 0.0009547861);
  helio_sect.ajouterDouble("q1", 0.530000000);
  helio_sect.ajouterDouble("q2", 0.082013072);
  helio_sect.ajouterDouble("q1point", 0.077336855);
  helio_sect.ajouterDouble("q2point", -0.044371494);
  helio_sect.ajouterDouble("pasintegration", 0.001);
  helio_sect.ajouterEntier("ralentissement", 0);
  helio_sect.ajouterEntier("periodeAff", 400);
  helio_sect.ajouterString("integrateur", choixInteg[1]);
  helio_sect.ajouterDouble("J0", 4.002783939123135);
  addEnsemble("Trajectoire heliocentrique SECTION", helio_sect);

  // triskel
  ListeValeurs triskel_rk4_pv = new ListeValeurs();
  triskel_rk4_pv.ajouterDouble( "mu", 0.0009547861);
  triskel_rk4_pv.ajouterDouble("q1", 0.5);
  triskel_rk4_pv.ajouterDouble("q2", 0.866025403784439);
  triskel_rk4_pv.ajouterDouble("q1point", -0.1);
  triskel_rk4_pv.ajouterDouble("q2point", 0.1);
  triskel_rk4_pv.ajouterDouble("pasintegration", 0.1);
  triskel_rk4_pv.ajouterEntier("ralentissement", 0);
  triskel_rk4_pv.ajouterEntier("periodeAff", 1);
  //triskel_rk4_pv.ajouterEntier("nbP", 100);
  triskel_rk4_pv.ajouterString("integrateur", choixInteg[0]);
  triskel_rk4_pv.ajouterDouble("J0", 2.9790468099885);
  addEnsemble("Triskel RK4_PV", triskel_rk4_pv);

  // triskel du RK4 symplectique
  ListeValeurs triskel_rk4_symp = new ListeValeurs();
  triskel_rk4_symp.ajouterDouble( "mu", 0.0009547861);
  triskel_rk4_symp.ajouterDouble("q1", 0.5);
  triskel_rk4_symp.ajouterDouble("q2", 0.866025403784439);
  triskel_rk4_symp.ajouterDouble("q1point", -0.1);
  triskel_rk4_symp.ajouterDouble("q2point", 0.1);
  triskel_rk4_symp.ajouterDouble("pasintegration", 0.01);
  triskel_rk4_symp.ajouterEntier("ralentissement", 0);
  triskel_rk4_symp.ajouterEntier("periodeAff", 10);
  //triskel_rk4_symp.ajouterEntier("nbP", 100);
  triskel_rk4_symp.ajouterString("integrateur", choixInteg[1]);
  triskel_rk4_symp.ajouterDouble("J0", 2.9790468099885);
  addEnsemble("Triskel RK4_SYMP", triskel_rk4_symp);
  
  // SECTION triskel du RK4 symplectique
  ListeValeurs triskel_sect = new ListeValeurs();
  triskel_sect.ajouterDouble( "mu", 0.0009547861);
  triskel_sect.ajouterDouble("q1", 0.5);
  triskel_sect.ajouterDouble("q2", 0.866025403784439);
  triskel_sect.ajouterDouble("q1point", -0.1);
  triskel_sect.ajouterDouble("q2point", 0.1);
  triskel_sect.ajouterDouble("pasintegration", 0.01);
  triskel_sect.ajouterEntier("ralentissement", 0);
  triskel_sect.ajouterEntier("periodeAff", 400);
  triskel_sect.ajouterString("integrateur", choixInteg[1]);
  triskel_sect.ajouterDouble("J0", 2.9790468099885);
  addEnsemble("Triskel SECTION", triskel_sect);

  // courbe autour de L4 et L5 avec rebond de la sonde
  ListeValeurs courbeL4L5 = new ListeValeurs();
  courbeL4L5.ajouterDouble( "mu", 0.1);
  courbeL4L5.ajouterDouble("q1", -0.8);
  courbeL4L5.ajouterDouble("q2", 0.0);
  courbeL4L5.ajouterDouble("q1point",  0.5);
  courbeL4L5.ajouterDouble("q2point", 0.2);
  courbeL4L5.ajouterDouble("pasintegration", 0.02);
  courbeL4L5.ajouterEntier("ralentissement", 0);
  courbeL4L5.ajouterEntier("periodAff", 1);
  //courbeL4L5.ajouterEntier("nbP", 100);
  courbeL4L5.ajouterString("integrateur", choixInteg[0]);
  courbeL4L5.ajouterDouble("J0", 3.039076);
  addEnsemble("Vitesses nulles autour de L4 et L5, avec rebonds", courbeL4L5);

  // courbe en C + trajectoire qui sort
  ListeValeurs courbeCetTraj = new ListeValeurs();
  courbeCetTraj.ajouterDouble( "mu", 0.1);
  courbeCetTraj.ajouterDouble("q1", -0.8);
  courbeCetTraj.ajouterDouble("q2", 0.0);
  courbeCetTraj.ajouterDouble("q1point",  0.012987562);
  courbeCetTraj.ajouterDouble("q2point", 0.0);
  courbeCetTraj.ajouterDouble("pasintegration", 0.1);
  courbeCetTraj.ajouterEntier("ralentissement", 0);
  courbeCetTraj.ajouterEntier("periodAff", 1);
  //courbeL4L5.ajouterEntier("nbP", 100);
  courbeCetTraj.ajouterString("integrateur", choixInteg[0]);
  courbeCetTraj.ajouterDouble("J0", 3.3289069);
  addEnsemble("Vitesses nulles en C, avec chappement de la sonde", courbeCetTraj);

  //courbe en 0 + sonde piege
  ListeValeurs courbeO = new ListeValeurs();
  courbeO.ajouterDouble( "mu", 0.1);
  courbeO.ajouterDouble("q1", 1.22);
  courbeO.ajouterDouble("q2", 0.0);
  courbeO.ajouterDouble("q1point",  0.012987562);
  courbeO.ajouterDouble("q2point", 0.0);
  courbeO.ajouterDouble("pasintegration", 0.1);
  courbeO.ajouterEntier("ralentissement", 0);
  courbeO.ajouterEntier("periodAff", 1);
  //courbeO.ajouterEntier("nbP", 100);
  courbeO.ajouterString("integrateur", choixInteg[0]);
  courbeO.ajouterDouble("J0", 3.476876869);
  addEnsemble("Vitesses nulles en O, avec la sonde pige", courbeO);

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//************************************************************//
//                    PARAMETRES DE SORTIE                    //
//************************************************************//
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  // pour le trac de la trajectoire dans le repere tournant et les coubers de vitesses nulles
  ParamOut q1 = new ParamOut();
  q1.type = "nombre";
  q1.label = "q1";
  addParamOut(q1);

  ParamOut q2 = new ParamOut();
  q2.type = "nombre";
  q2.label = "q2";
  addParamOut(q2);

  // Pour les courbes de vitesse nulles seulement
  ParamOut q11 = new ParamOut();
  q11.type = "nombre";
  q11.label = "q11";
  addParamOut(q11);

  ParamOut q21 = new ParamOut();
  q21.type = "nombre";
  q21.label = "q21";
  addParamOut(q21);

  ParamOut q12 = new ParamOut();
  q12.type = "nombre";
  q12.label = "q12";
  addParamOut(q12);

  ParamOut q22 = new ParamOut();
  q22.type = "nombre";
  q22.label = "q22";
  addParamOut(q22);

  // Pour tracer la valeur relative de l'integrale de Jacobi
  ParamOut jacobiRelatif = new ParamOut();
  jacobiRelatif.type = "nombre";
  jacobiRelatif.label = "jacobiRelatif";
  addParamOut(jacobiRelatif);

  ParamOut appel = new ParamOut();
  appel.type = "nombre";
  appel.label = "appel";
  addParamOut(appel);
   
  // Pour tracer le trajectoire dans le repere fixe
  ParamOut q1fixe = new ParamOut();
  q1fixe.type = "nombre";
  q1fixe.label = "q1fixe";
  addParamOut(q1fixe);

  ParamOut q2fixe = new ParamOut();
  q2fixe.type = "nombre";
  q2fixe.label = "q2fixe";
  addParamOut(q2fixe);
    
  // Pour donner la valeur de Jacobi initial
  ParamOut info = new ParamOut();
  info.type = "string";
  info.label = "jacobiInitial";
  addParamOut(info);
  
  // Pour tracer la section de Poincar
  ParamOut p1 = new ParamOut();
  p1.type = "nombre";
  p1.label = "p1";
  addParamOut(p1);
  
  ParamOut p2 = new ParamOut();
  p2.type = "nombre";
  p2.label = "p2";
  addParamOut(p2);


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//************************************************************//
//                        GRAPHIQUES                          //
//************************************************************//
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// pour la realisation du graphe, certains parametres vont etre modifies au cours du programme, 
// par exemle la connection des points et le style de points, ceci dans le but de differencier 
// les masses, les points de Lagrange, la trajectoire de la sonde.
    
//=======================================================
//      PLOTLIVE : TRAJECTOIRE DANS REPERE TOURNANT
//=======================================================

// le nom de ce plot est "graphe", a retenir pour modifier ensuite certains parametres

  graphe.type = "plotlive";
  graphe.titre = "Repre tournant";
  String[][] pparam = {{"q1", "q2"}};
  graphe.params = pparam;
  graphe.carre = true;

  // initialisation de la connection, du style de points: pour les masses et les points de Lagrange
  graphe.points = "dots";
  graphe.connecter = false;

  addAffichage(graphe);
    
//=================================================
//     PLOTLIVE : TRAJECTOIRE DANS REPERE FIXE
//=================================================

// le nom de ce plot est "fixe", a retenir pour modifier ensuite certains parametres
/*
  fixe.type = "plotlive";
  fixe.titre = "Trajectoire live dans le repre fixe";
  String[][] pparamf = {{"q1fixe", "q2fixe"}};
  fixe.params = pparamf;
  fixe.carre = true;

  // initialisation de la connection, du style de points: pour les masses et les points de Lagrange
  fixe.points = "none";
  fixe.connecter = false;

  // ajustement automatique de la fenetre graphique
  //fixe.ajuster = true;
  addAffichage(fixe);
*/
 
//=======================================
//     COURBES DES VITESSES NULLES
//=======================================
  
// le nom de ce plot est "courbeVitesse", a retenir pour modifier ensuite certains parametres

  //   Affichage courbeVitesse = new Affichage();
  courbeVitesse.type = "plotlive";
  courbeVitesse.titre = "Courbe de vitesse nulle";
  String[][] pparamc = {{"q1", "q2"},{"q11","q21"},{"q12","q22"}};
  courbeVitesse.params = pparamc;
  courbeVitesse.carre = true;

  // initialisation de la connection des points : pour les masses et les points de Lagrange
  courbeVitesse.points = "dots";
  courbeVitesse.connecter = false;

  addAffichage(courbeVitesse);
  
//=======================================
//         SECTION DE POINCARE
//=======================================

//le nom de ce plot est "section", a retenir pour modifier ensuite certains parametres

  sectionTraj.type = "multiplot";
  sectionTraj.titre = "Section de Poincar et variations de J0";
  sectionTraj.sousAff = new Affichage[2];
  
  // Section
  section1.type = "plotlive";
  section1.titre = "Section de Poincar  q2 = 0";
  String[][] pparamsp1 = {{"q1", "p1"}};
  section1.params = pparamsp1;
  //section1.carre = true;
  sectionTraj.sousAff[0] = section1;
  // initialisation de la connection des points : pour les masses et les points de Lagrange
  section1.points = "points";
  section1.connecter = false;
  
  // Trajectoire repre tournant
  section2.type = "plotlive";
  section2.titre = "jacobi Relatif : (J-J0) / J0";
  String[][] pparamsp2 = {{"appel", "jacobiRelatif"}};
  section2.params = pparamsp2;
  sectionTraj.sousAff[1] = section2;
  // initialisation de la connection des points : pour les masses et les points de Lagrange
  section2.points = "points";
  section2.connecter = true;

  addAffichage(sectionTraj);

//==========================================
//    MULTI-PLOT : TRAJECTOIRE + JACOBI
//==========================================

  //   Affichage multi = new Affichage();
  multi.type = "multiplot";
  multi.titre = "Trajectoire et integrale de Jacobi";
  multi.sousAff = new Affichage[2];

  // le nom du graphe qui concerne la trajectoire est m1, pour la modification des parametres
  //   Affichage m1 = new Affichage();
  m1.type = "plotlive";
  m1.titre = "Trajectoire";
  String[][] pparamsm1 = {{"q1", "q2"}};
  m1.params = pparamsm1;
  m1.carre = true;
  multi.sousAff[0] = m1;

  // initialisation de la connection et du style de points: masses et points de Lagrange
  m1.connecter = false;
  m1.points = "dots";

  // ajustement automatique de la fenetre graphique
  //m1.ajuster = true;

  // le nom du graphe qui concerne l'integrale de Jacobi est m2, les parametres ne doivent pas changer.
  //   Affichage m2 = new Affichage();
  m2.type = "plotlive";
  m2.titre = "Valeur relative de l'integrale de Jacobi";
  String[][] pparamsm2 = {{"appel","jacobiRelatif"}};
  m2.params = pparamsm2;
  multi.sousAff[1] = m2;

  // initialisation de la connection et du style de points
  m2.connecter = true;
  m2.points = "points";

  // ajustement automatique de la fenetre graphique
  //m2.ajuster = true;

  addAffichage(multi);

//===============================================
//      MULTI-PLOT : REPERES TOURNANT + FIXE
//===============================================

  //   Affichage toufix = new Affichage();
  toufix.type = "multiplot";
  toufix.titre = "Repres tournant et fixe";
  toufix.sousAff = new Affichage[2];

  // le nom du plot qui concerne la trajectoire dans le repere tournant est tf1, pour la modification des parammetres
  tf1.type = "plotlive";
  tf1.titre = "Dans le repere tournant";
  String[][] pparamstf1 = {{"q1", "q2"}};
  tf1.params = pparamstf1;
  tf1.carre = true;    
  toufix.sousAff[0] = tf1;

  // initialisation de la connection et du style de points, masses et points de Lagrange
  tf1.connecter = false;
  tf1.points = "dots";

  // ajustement automatique de la fenetre graphique
  //tf1.ajuster = true;

  // le nom du plot qui concerne la trajectoire dans le repere fixe est tf2
  tf2.type = "plotlive";
  tf2.titre = "Dans le repere fixe";
  String[][] pparamstf2 = {{"q1fixe", "q2fixe"}};
  tf2.params = pparamstf2;
  tf2.carre = true;
  toufix.sousAff[1] = tf2;

  // attention, initialisation diffrente car pour l'instant, on ne trace pas les masses et les points de Lagrange !
  // initialisation de la connection et du style de points
  tf2.connecter = false ;
  tf2.points = "none";

  // ajustement automatique de la fenetre graphique
  //tf2.ajuster = true;

  addAffichage(toufix);

  appletDoc("trois-corps-applet-2007/trois-corps-applet-2007.xml");
}


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// ************************************************************//
//                 CALCULELIVE : initialisation                //
// ************************************************************//
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


public void initCalculLive(ListeValeurs in) throws SimuException {

   // ajustement automatique des fenetres graphiques
   String ajust = in.lireString("ajustement");
   if (ajust.equals("on")) {
       graphe.ajuster = true;
       //fixe.ajuster = true;
       m1.ajuster = true;
       m2.ajuster = true;
       tf1.ajuster = true;
       tf2.ajuster = true;
       courbeVitesse.ajuster = true;
       section1.ajuster = true;
       section2.ajuster = true;
   }
   else {
       graphe.ajuster = false;
       //fixe.ajuster = false;
       m1.ajuster = false;
       m2.ajuster = true;
       tf1.ajuster = false;
       tf2.ajuster = false;
       courbeVitesse.ajuster = false;
       section1.ajuster = false;
       section2.ajuster = true;
   }

   // section de Poincar ?
   sectionPoincare = in.lireString("sectionP");
   nbPtsTrajP = in.lireEntier("periodeAff");

   // courbe de vitesse nulle ?
   courbe = in.lireString("vitessenulle");
   
   //courbe de vitesse nulle et trajectoire ?
   courbeTrajectoire = in.lireString("vitessenulleEtTrajectoire") ;

   // initialisation des diverses valeurs necessaires au calcul
   q1Initial = in.lireDouble("q1");
   q2Initial = in.lireDouble("q2");

   q1dotInitial = in.lireDouble("q1point");
   q2dotInitial = in.lireDouble("q2point");

   p1Initial = q1dotInitial - q2Initial;
   p2Initial = q2dotInitial + q1Initial;

   // pas d'integration
   deltaT = in.lireDouble("pasintegration");

   // temporisation
   dodo = in.lireEntier("ralentissement");
  
   // periode d'affichage des points
   periodeAffichage = in.lireEntier("periodeAff");

   // choix de l'intgrateur
   typeInteg = in.lireString("integrateur");
   relaxation = 0.9;
   
   /*// section de Poincar
   p10min = in.lireDouble("MinP10");
   p10max = in.lireDouble("MaxP10");
   q10min = in.lireDouble("MinQ10");
   q10max = in.lireDouble("MaxQ10");
   pasp10 = in.lireDouble("DP10");
   pasq10 = in.lireDouble("DQ10");
   jacobiP = in.lireDouble("J0");*/

   // valeur du mu
   mu = in.lireDouble("mu");

  // tableau transmis au calcul pour l'initialisation, contient q1 q2 q1dot q2dot
  double[] initialisation = new double[4];
  initialisation[0] = q1Initial;
  initialisation[1] = q2Initial;
  initialisation[2] = p1Initial;
  initialisation[3] = p2Initial;
  voyager = initialisation;

  // mu sous forme de tableau, envoye au calcul, pour une histoire de generalisation il a besoin d'un tableau
  mutab = new double[1];
  mutab[0] = mu;

  // ********* ---pour le calcul de la trajectoire--- *********
  //monIntegrateur = new Integrateur(4,mutab,"EqMvmtPblmTroisCorps");
  monIntegrateur = new Integrateur(4,mutab,fEqMvmt);

  // ********* ---pour la calcul de l'integrale de Jacobi--- *********
  mouvement = UsineEqMvmt.nouveau(fEqMvmt);
  jacobiInitial = mouvement.intPrem(voyager,mutab);
  
  //********* ---pour le calcul de la section de Poincar --- *********
  jacobiP = in.lireDouble("J0");
  pasp10 = 0.1*Math.abs(p1Initial);//0.001
  pasq10 = 0.01*Math.abs(q1Initial);//0.01
  if(p1Initial<0){
	  p10min = p1Initial;
	  p10max = -p1Initial;
  }
  else{
	  p10min = -p1Initial;
	  p10max = p1Initial;
  }
  //p10min = p1Initial;
  //p10max = -p1Initial;//p1Initial + 3.5*Math.abs(p1Initial); // choix arbitraire...
  q10min = q1Initial;
  q10max = q1Initial;// + 0.5*Math.abs(q1Initial); // pour que q10 soit changer par l'utilisateur
  sectionDePoincare = new SectionPoincare(jacobiP, pasp10, pasq10, p10min, p10max, q10min, q10max);
  surfaceSection = sectionDePoincare.choixSection();

  // ********* ---courbe de vitesse nulle--- *********
  vitesseNulle = new CourbeVitesse0(jacobiInitial,mu,0.05);//implementer Courbevitesse0
  qI = new double[2];//position du point initial de la courbe de vitesse nulle pour q1 et q2
  qI[0] = 0.0;
  qI[1] = 0.0;
  qI1 = new double[2];//position du point initial de la courbe de vitesse nulle pour q11 et q21
  qI1[0] = 0.0;
  qI1[1] = 0.0;
  qI2 = new double[2];//position du point initial de la courbe de vitesse nulle pour q12 et q22
  qI2[0] = 0.0;
  qI2[1] = 0.0;

  stop = new double[3];//variable d'arret des courbes de vitesses nulles
  stop[0] = 0.0;
  stop[1] = 0.0;
  stop[2] = 0.0;
  trajV0 = 0;
  
  //initialisation des parametres q, qq1 et qq2
  q=qI;
  qq1=qI1;
  qq2=qI2;
  
  origin = new double [2];//point origine vitesse nulle
  origin1 = new double [2];//point origine vitesse nulle
  origin2 = new double [2];//point origine vitesse nulle
  
  // initialisation du compteur
  appel = 0;
  
  // Section de Poincar
  ptsTrajP = 0;
  appelP = 2000;		// nb max d'appel sans passer par la section de Poincare
  voyagerAll = new double[12];
  compteurq10 = 0;

  // changement de style de points: la c'est pour quand on joue plusieurs fois a relancer l'applet, pour reinitialiser correctement !
  graphe.points = "dots";
  //fixe.points = "none";
  m1.points = "dots";
  m2.points = "points";
  tf1.points = "dots";
  tf2.points = "none";
  courbeVitesse.points = "dots";
  section1.points = "points";
  section2.points = "points";

}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//************************************************************//
//                 CALCULELIVE : trajectoire                  //
//************************************************************//
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

public ListeValeurs calculLive() throws SimuException {

   // indication de la precision
   double precision = 1.0e-10;

  // tableau qui va contenir tous les parametres de sortie   
  ListeValeurs out = new ListeValeurs();

  PointFixe pointDeLagrange;   // pour les points de Lagrange
  double q1, q2;         // parametres de sortie donnes dans le repere tournant
  double p1, p2;         // moments conjugues 
  //  double q1dot, q2dot;   // vitesses de sortie dans le repere tournant
  double q1fixe, q2fixe;  // parametres de sortie pour le repere fixe
  double jacobi;  // integrale de jacobi absolue
  double jacobiRelatif;  // valeur relative de l'intergrale de Jacobi
  double q11, q21, q12, q22 ;   //parametres de sorties supplmentaire pour les courbes de vitesse Nulles.

//********initialisation  zero des parametres de sorties supplmentaire pour les courbes de vitesse Nulles (plot un point au niveau du centre de masse sous certaines conditions)
  q11=0.0;
  q21=0.0;
  q12=0.0;
  q22=0.0;

  // on a maintenant deux cas : soit c'est la courbe de vitesse nulle qui est demande et dans ce cas, on va  "else", soit elle n'est pas demandee, et on fait ce qui suit...

//==================================================================================
//                         PREMIER CAS : TRAJECTOIRES
//==================================================================================  
  
//  ****  si la courbe de vitesse nulle n'est pas demandee  ****  //

if ((courbe.equals("off"))&&(courbeTrajectoire.equals("off"))&&(sectionPoincare.equals("off"))) {
    choix="traj" ;
    }
else if(((courbe.equals("on"))||(courbeTrajectoire.equals("on")))&&(sectionPoincare.equals("off"))&&(trajV0==0)) {
    choix="vite";
}
else if(((courbe.equals("on"))||(courbeTrajectoire.equals("on")))&&(sectionPoincare.equals("off"))&&(trajV0==1)) {
	choix="traj" ;
}
else{
	choix="sect";
}

if (choix.equals("traj")) {

/////////////////////////////////////////////////////////////////////////
// pour tracer les graphes et faire le calcul:                         //
// * appel 1 = position de la masse 1                                  //
// * appel 2 = position de la masse 2                                  //
// * pour les 5 appels suivants (2 < appel < 8 ): points de lagrange   //
// * appel == 8: position initiale de la sonde                         //
// * appel == 9: initialisation du calcul de trajectoire               //
// * appel > 9: reste de la trajectoire                                //
/////////////////////////////////////////////////////////////////////////

// le compteur evolue avec le nombre d'appel du calcul
appel++;

// ********* ---premiere masse--- *********

    if (appel == 1) {
        // pour la trajectoie dans le repere tournant
        q1 = - mu;
        q2 = 0;
        p1 = p1Initial;
        p2 = p2Initial;
        
        // pour la trajectoire dans le repere fixe
        q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
        q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.); 
        
        // pour la calcul de l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
           
        // connection entre les points
        graphe.connecter = false;
        //fixe.connecter = false;
        m1.connecter = false;
        m2.connecter = true;
        tf1.connecter = false;
        tf2.connecter = false;
        courbeVitesse.connecter = false;
    }

// ********* ---seconde masse--- *********

    else if (appel == 2) {
        // pour la trajectoire dans le repere tournant
        q1 = 1.0 - mu;
        q2 = 0;
        p1 = p1Initial;
        p2 = p2Initial;
        
        // pour la trajectoire dans le repere fixe
        q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
        q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);
        
        // pour l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
        
        // pour changer la couleur des points, entre les masses et les points de Lagrange
        out.ajouterString("fin", "courbe");    
    } 


// ********* --- cinq points de lagrange--- *********

    else if (2 < appel && appel < 8) {

        pointDeLagrange = new PointFixe(mutab);
        int numeroDeLagrange = appel - 2;
        pointDeLagrange.determineLagrange(numeroDeLagrange, 1e-13);
        
        // pour le repere tournant     
        q1 = pointDeLagrange.getPosition("q1");
        q2 = pointDeLagrange.getPosition("q2");
        p1 = p1Initial;
        p2 = p2Initial;
        
        // pour le repere fixe
        
        q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
        q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);
        
        // pour l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
        
        if (appel == 7) { 
            // change de nouveau la couleur des points, entre les points de Lagrange et la trajectoire
            out.ajouterString("fin", "courbe");
            
            // changement de caracteristiques pour les plots
            // pour passer  la trajectoire dans les divers graphes
            graphe.connecter = true;
            graphe.points = "points";
            //fixe.connecter = true;
            //fixe.points = "points";
            m1.connecter = true;
            m1.points = "points";
            tf1.connecter = true;
            tf1.points = "points";
            tf2.connecter = true;
            tf2.points = "points";
            courbeVitesse.connecter = true;
            courbeVitesse.points = "points";
            }
        }

        // ********* ---position initiale de la sonde--- *********
        
    else if (appel == 8) {
        // pour le repere tournant
        q1 = q1Initial;
        q2 = q2Initial;
        p1 = p1Initial;
        p2 = p2Initial;
        
        // pour l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
        
        // pour le repere fixe
        q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
        q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);
    }


// ********* ---premier appel de l'integrateur: premier point trace--- *********

    else if (appel == 9) {
    	if (typeInteg == choixInteg[0]){
			voyager = monIntegrateur.rk4pv(voyager, deltaT, precision); //integrateur RK4_PV
		}
		else {
			voyager = monIntegrateur.rk4sympf(voyager, deltaT, precision, 0.9); //integrateur RK4_SYMP
		}
        
        // dans le repre tournant       
        q1 = voyager[0];
        q2 = voyager[1];
        p1 = voyager[2];
        p2 = voyager[3];
         
        // dans le repere fixe
        q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
        q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);
        
        // calcul de l'integrale de jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
    }

// ********* ---toutes les autres positions de la sonde--- *********

    else {
        // temporisation
        dormir(dodo);
        
        if (typeInteg == choixInteg[0]){
        	//periodeAffichage=1;
			for(int k=0;k<periodeAffichage;k++){
				voyager = monIntegrateur.rk4pv(voyager, deltaT, precision,true); //integrateur RK4_PV
				appel ++;
			}
		}
		else {
			//periodeAffichage=10;
			// Pour pouvoir afficher 1 point sur 10 avec RK4 symplectique
			// Le pas d'integration tant 10 fois plus grand que pour le RK4_PV, 
			//cela permet de tracer des points sur le graphique avec la mme frquence
			for(int k=0;k<periodeAffichage;k++){
				voyager = monIntegrateur.rk4sympf(voyager, deltaT, precision, relaxation); //integrateur RK4_SYMP 
				appel ++;
			}
		}
        
        // pour le repere tournant     
        q1 = voyager[0];
        q2 = voyager[1];
        p1 = voyager[2];
        p2 = voyager[3];
        
        // pour le repere fixe
        q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
        q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);
        
        // calcul de l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
    }

    // ********* ---remplissage du tableau de parametres de sortie---*********
    
    // pour la trajectoire dans le repere tournant
    out.ajouterDouble("q1", q1);
    out.ajouterDouble("q2", q2);
    //pour le graphique des courbes de vitesses nulles
    out.ajouterDouble("q11", q11);
    out.ajouterDouble("q21", q21);
    out.ajouterDouble("q12", q12);
    out.ajouterDouble("q22", q22);
    // pour l'integrale de Jacobi
    out.ajouterDouble("jacobiRelatif", jacobiRelatif);
    out.ajouterDouble("appel", appel);
    // pour la trajectoire dans le repere fixe
    out.ajouterDouble("q1fixe", q1fixe);
    out.ajouterDouble("q2fixe", q2fixe);
    out.ajouterDouble("p1", p1);
    out.ajouterDouble("p2", p2);
    // pour la valeur de l'integrale de jacobi    
    out.ajouterString("jacobiInitial", "valeur initiale de l'intgrale de Jacobi " + jacobiInitial);
    //out.ajouterString("jacobiInitial", "q2 vaut = " + q1Initial + "CHOIX vaut : " + choix);
    return out;
}

//==================================================================================
//                       DEUXIEME CAS : VITESSES NULLES
//==================================================================================

//  ****  si la courbe de vitesse nulle est demandee  ****   //

else if (choix.equals("vite")) {

    int cas=0;    //paramtres permettant de savoir quels types de courbes de vitesses nulles nous avons dans les conditions du problme.
    
    //Calcul de la position et la vitesse en L3
    pointDeLagrange = new PointFixe(mutab);
    int numeroDeLagrange = 3;
    pointDeLagrange.determineLagrange(numeroDeLagrange, 1e-13);
    double q1l3 ;
    q1l3 = pointDeLagrange.getPosition("q1");   
           
    double fl3;
    fl3=(q1l3*q1l3+2*((1-mu)/(Math.abs(q1l3+mu))+(mu)/(Math.abs(q1l3+mu-1)))-jacobiInitial);
        
    //Calcul de la position et la vitesse en L2        
    numeroDeLagrange = 2;
    pointDeLagrange.determineLagrange(numeroDeLagrange, 1e-13);
    double q1l2 ;
    q1l2 = pointDeLagrange.getPosition("q1");   
           
    double fl2;
    fl2=(q1l2*q1l2+2*((1-mu)/(Math.abs(q1l2+mu))+(mu)/(Math.abs(q1l2+mu-1)))-jacobiInitial);

    //Calcul de la position et la vitesse en L1
    numeroDeLagrange = 1;
    pointDeLagrange.determineLagrange(numeroDeLagrange, 1e-13);
    double q1l1 ;
    q1l1 = pointDeLagrange.getPosition("q1");   
          
    double fl1;
    fl1=(q1l1*q1l1+2*((1-mu)/(Math.abs(q1l1+mu))+(mu)/(Math.abs(q1l1+mu-1)))-jacobiInitial);

    if((jacobiInitial>(3-mu+mu*mu))&&(fl3>0)){//cas des courbes de vitesses nulles symetriques autours de L4 et L5
        cas = 1;
        }

    if((fl3<0)&&(fl2>0)) {//cas des courbes de vitesses nulles en "C"
        cas=2;
        }
    
    if((fl2<0)&&(fl1>0)) {//cas des courbes de vitesses nulles en "O"
        cas = 3;
        }

    if(fl1<0) {//cas des courbes de vitesses nulles en fermes et troues autours de la premire et de la deuxime masse
        cas = 4;
        }
// le compteur evolue avec le nombre d'appel du calcul

    if(stop[2]==1.0) {
        appel=0 ;  
    }

    else {
        appel++;

        //-Debugging
        //System.out.println("incremente appel");
        //-Debugging
    }
    
    //////////////////////////////////////////////////
    // appel 0: fin des calculs                     //
    // appel 1: premiere masse                      //
    // appel 2: seconde masse                       //
    // appels 3 a 7: points de Lagrange             //
    // appel 8: premier point de la courbe          //
    // reste des appels: courbe de vitesse nulle    //
    //////////////////////////////////////////////////
    
    //********* ---fin des calculs---********
    //N.B.:arrive bien  la fin
    
    if (appel==0) {

        //-Debugging
        //System.out.println("appel zero");
        //-Debugging

        if((courbeTrajectoire.equals("off"))&&(courbe.equals("on"))) {
            return null;
            }
      
        if((courbeTrajectoire.equals("on"))&&(courbe.equals("off"))) {
            choix = "traj";//modifie la valeur de choix, permettant de commencer la trajectoire  la prochaine itration
            trajV0=1;
            }
        }
    
    // ********* ---premiere masse--- *********

    if (appel == 1) {

        //-Debugging
        //System.out.println("appel 1");
        //-Debugging
        
        // pour la trajectoie dans le repere tournant
        q1 = - mu;
        q2 = 0;
        p1 = p1Initial;
        p2 = p2Initial;
        
        // pour la trajectoire dans le repere fixe
        q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
        q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.); 
        
        // pour la calcul de l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;

        //Pour les vitesses nulles (rajoute un point au centre de masse si l'affichage courbeVitesse est choisi)
        q11=0.0;
        q21=0.0;
        q12=0.0;
        q22=0.0;
        
        // connection entre les points
        graphe.connecter = false;
        //fixe.connecter = false;
        m1.connecter = false;
        m2.connecter = true;
        tf1.connecter = false;
        tf2.connecter = false;
        courbeVitesse.connecter = false;                
    }

    // ********* ---seconde masse--- *********
    
    else if (appel == 2) {
    
        //-Debugging
        //System.out.println("appel 2");
        //-Debugging
        
        // pour la trajectoire dans le repere tournant
        q1 = 1.0 - mu;
        q2 = 0;
        p1 = p1Initial;
        p2 = p2Initial;
        
        // pour la trajectoire dans le repere fixe
        q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
        q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);
        
        // pour l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
        
        //Pour les vitesses nulles (rajoute un point au centre de masse si l'affichage courbeVitesse est choisi)  
        q11=0.0;
        q21=0.0;
        q12=0.0;
        q22=0.0;

        // pour changer la couleur des points, entre les masses et les points de Lagrange
        out.ajouterString("fin", "courbe");

        // connection entre les points
        graphe.connecter = false;
        //fixe.connecter = false;
        m1.connecter = false;
        m2.connecter = true;
        tf1.connecter = false;
        tf2.connecter = false;
        courbeVitesse.connecter = false;        
    } 


    // ********* --- cinq points de lagrange--- *********
    
    else if (2 < appel && appel < 8) {
    
        //-Debugging
        //System.out.println("appel 3 a 7");
        //-Debugging
        
        pointDeLagrange = new PointFixe(mutab);
        numeroDeLagrange = appel - 2;
        pointDeLagrange.determineLagrange(numeroDeLagrange, 1e-13);
        
        // pour le repere tournant     
        q1 = pointDeLagrange.getPosition("q1");
        q2 = pointDeLagrange.getPosition("q2");
        p1 = p1Initial;
        p2 = p2Initial;
        
        // pour le repere fixe
        q1fixe = q1*Math.cos(0.) - q2*Math.sin(0.);
        q2fixe = q1*Math.sin(0.) + q2*Math.cos(0.);
        
        // pour l'integrale de Jacobi
        jacobi = mouvement.intPrem(voyager,mutab);
        jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
        
        //Pour les vitesses nulles (rajoute un point au centre de masse si l'affichage courbeVitesse est choisi)  
        q11=0.0;
        q21=0.0;
        q12=0.0;
        q22=0.0;

        if (appel == 7) { 
            // change de nouveau la couleur des points, entre les points de Lagrange et la trajectoire
            out.ajouterString("fin", "courbe");
            
            // changement de caracteristiques pour les plots
            // pour passer  la trajectoire dans les divers graphes
            graphe.connecter = true;
            graphe.points = "points";
            //fixe.connecter = true;
            //fixe.points = "points";
            m1.connecter = true;
            m1.points = "points";
            tf1.connecter = true;
            tf1.points = "points";
            tf2.connecter = true;
            tf2.points = "points";
            courbeVitesse.connecter = true;
            courbeVitesse.points = "points";
        }

    }
    
    else if (appel == 8) {
    
        //-Debugging
        //System.out.println("appel 8");
        //-Debugging

        if(cas == 1) {//origin = l4
            origin[0] = 1.0/2.0 - mu;
            origin[1] = Math.pow(3.0, 0.5)/2.0;
            //-Debugging
            //System.out.println("origin = " +origin[0] + origin[1]);
            //-Debugging
            qI1 = vitesseNulle.pointV0(origin);
            qq1=qI1;
        
            q11=qI1[0];
            q21=-qI1[1];
            }
        
        if(cas == 2) {//origin = l3
            origin[0] = q1l3;
            origin[1] = 0.0;
            //-Debugging
            //System.out.println("origin = " +origin[0] + origin[1]);
            //-Debugging
            }
            
         if(cas == 3) {//origin=L3 et origin1 = centre de masse
            origin[0] = q1l3;
            origin[1] = 0.0;
            
            
            origin1[0] = 0.0;
            origin1[1] = 0.0;
            
            qI1 = vitesseNulle.pointV0(origin1);
            qq1=qI1;
        
            q11=qI1[0];
            q21=qI1[1];

            }

         if(cas == 4) {//origin=L3, origin1 = centre de masse et origin2 = deuxime masse
            origin[0] = q1l3;
            origin[1] = 0.0;
            
            
            origin1[0] = 0.0;
            origin1[1] = 0.0;
            
            origin2[0] = 1.0 - mu;
            origin2[1] = 0.0;
            
            qI1 = vitesseNulle.pointV0(origin1);
            qq1=qI1;
        
            q11=qI1[0];
            q21=qI1[1];

            qI2 = vitesseNulle.pointV0(origin2);
            qq2=qI2;
        
            q12=qI2[0];
            q22=qI2[1];

            //-Debugging
            //System.out.println("qI2 = " +qI2);
            //-Debugging
            }

  //dans tous les cas
        qI = vitesseNulle.pointV0(origin);
        q=qI;
        
        q1=qI[0];
        q2=qI[1];
        p1 = p1Initial;
        p2 = p2Initial;
        
        q1fixe = 0.0;
        q2fixe = 0.0;
        
        jacobiRelatif = 0.0; 
    }

    else {  
    
        //-Debugging
        //System.out.println("appel autres");
        //-Debugging
        

                
        if(jacobiInitial<=(3-mu+mu*mu)) {//Pour ces valeurs de mu et du Jacobien il n'y a pas de courbes de vitesses nulles
            //-Debugging
            //System.out.println("cas 0");
            //-Debugging

            return null;
        }

        else {       
            //-Debugging
            //System.out.println("etape 2");
            //-Debugging
            
            
            double[] param = new double[2];
            
            param[0]=jacobiInitial;
            param[1]=mu;
            
            q = vitesseNulle.courbeV0(q);
            q1=q[0];
            q2=q[1];
            p1 = p1Initial;
            p2 = p2Initial;
        
            if (cas==1) {//cas des courbes de vitesse nulle symetriques autours de L4 et L5
                q11=q[0];
                q21=-q[1];
                
                q12=0.0;
                q22=0.0;
                
                stop = vitesseNulle.angle(stop,origin,q); 
                }

            if (cas==2) {//cas des courbes de vitesses nulles en "C"
                q11=0.0;
                q21=0.0;
                
                q12=0.0;
                q22=0.0;
                
                stop = vitesseNulle.angle(stop,origin,q); 
                }

             if (cas==3) {//cas des courbes de vitesses nulles en "O"
                          
                 
                 qq1 = vitesseNulle.courbeV0(qq1);
                 q11=qq1[0];
                 q21=qq1[1];
              
                 q12=0.0;
                 q22=0.0;
                             
                 stop = vitesseNulle.angle(stop,origin,q);
                 }     

             if (cas==4) {//cas des courbes de vitesses nulles en fermes et troues autours de la premire et de la deuxime masse  
                                         
                 qq1 = vitesseNulle.courbeV0(qq1);
                 q11=qq1[0];
                 q21=qq1[1];
                                 
                 qq2 = vitesseNulle.courbeV0(qq2);
                 q12=qq2[0];
                 q22=qq2[1];
                 
                 stop = vitesseNulle.angle(stop,origin,q);
                 }     
            }   
        

        q1fixe = 0.0;
        q2fixe = 0.0;
        
        jacobiRelatif = 0.0;     
    }

    // remplissage du tableau de parametres de sortie
    // pour la position des courbes de vitesse nulles
    out.ajouterDouble("q1", q1);
    out.ajouterDouble("q2", q2);
    out.ajouterDouble("q11", q11);
    out.ajouterDouble("q21", q21);
    out.ajouterDouble("q12", q12);
    out.ajouterDouble("q22", q22);
    // pour l'integrale de Jacobi
    out.ajouterDouble("jacobiRelatif", jacobiRelatif);
    out.ajouterDouble("appel", appel);
    // pour la position de la sonde dans le repere fixe
    out.ajouterDouble("q1fixe", q1fixe);
    out.ajouterDouble("q2fixe", q2fixe);
    out.ajouterDouble("p1", p1);
    out.ajouterDouble("p2", p2);
    // pour avoir une ide de la valeur initiale de l'integrale de Jacobi
    out.ajouterString("jacobiInitial", "valeur initiale de l'intgrale de Jacobi " + jacobiInitial);
    //out.ajouterString("jacobiInitial", "choix = " + choix + "stop = " + stop[2]);
    //message d'info
    // out.ajouterString("info", message);

    return out;
}

//==================================================================================
//                     TROISIEME CAS : SECTION POINCARE
//==================================================================================

else if (choix.equals("sect")) {
	
	// le compteur evolue avec le nombre d'appel du calcul
	appel++;
	if(appel == 1){		
		double[] parametresSection = new double[6];
		double[] retourSolP = new double[3];
		
		//-----------------------------------------------------------
		// Premier appel : on intialise tout
		//-----------------------------------------------------------
		// intialisation des paramtres (p10max, p10min, pasP10...) de la section
		parametresSection = sectionDePoincare.paramSection();
		pasp10 = parametresSection[0];
		pasq10 = parametresSection[1];
		p10min = parametresSection[2];
		p10max = parametresSection[3];
		q10min = parametresSection[4];
		q10max = parametresSection[5];
		
		p10 = p10min;
		q10 = q10min;
		jacobiInitial = jacobiP;
		//q2Initial = 0.;
		
		retourSolP = sectionDePoincare.calculp2H0(q10, q2Initial, p10, mu);
		p20a = retourSolP[0];
		p20b = retourSolP[1];
		solutionP = (int) retourSolP[2];
		
		// Tant qu'il n'y a pas de solution pour la varit d'nergie choisie 
		while (solutionP == 0){
			// on incrmente q10 si infrieut  q10max.
			if(q10 >= q10max){
				compteurq10++;
				q10 = q10min;
			}
			else{
				q10 = q10 + pasq10;
			}
			// incrmente p10 et/ou q10 si p10 = p10max, et/ou si q10 = q10max 
			// on incrmente p10 si q10 a atteint q10max
			if(p10 >= p10max){
				return null;
			}
			else if(p10 < p10max){
				p10 = p10min + compteurq10*pasp10;
			}
			
			retourSolP = sectionDePoincare.calculp2H0(q10, q2Initial, p10, mu);
			p20a = retourSolP[0];
			p20b = retourSolP[1];
			solutionP = (int) retourSolP[2];
		}
		
	    // dans le repre tournant
		q1 = q10;
		q2 = q2Initial; // on se place  q20 = 0 afin d'tre sr que les
						// trajectoires passent pas q2 = 0. A CHANGER pour
						// GENERALISER  des surfaces de section diffrentes
						// de q2 = 0.
		p1 = p10;
		p2 = p20a;
		
		// initialisation des voyagers
		voyagerAll[0] = q1;
		voyagerAll[1] = q2;
		voyagerAll[2] = p1;
		voyagerAll[3] = p20a;
		voyagerAll[4] = q1;
		voyagerAll[5] = q2;
		voyagerAll[6] = p1;
		voyagerAll[7] = p20b;
		voyagerAll[8] = appel;
		voyagerAll[9] = 0.;
		voyagerAll[10] = 0.;
		voyagerAll[11] = 0.;
		//-----------------------------------------------------------

		//-----------------------------------------------------------
		// On intgre pour aller au prochain point de la section car 
		// sinon trace un point n'appartenant pas  la section
		//-----------------------------------------------------------
		voyagerAll = sectionDePoincare.calculVoyager(voyagerAll, mu, fEqMvmt, deltaT, precision, surfaceSection, appelP);
		solutionP = (int) voyagerAll[9];
		solutionPa = (int) voyagerAll[10];
		solutionPb = (int) voyagerAll[11];
		
		if (solutionP == 1){
			// Compteur de points tracs
			ptsTrajP ++;
			// Si pas de solution pour voyager(p20a)	
			if((solutionPa == 0) && (solutionPb == 1)){
			    // dans le repre tournant
				q1 = voyagerAll[4];
				q2 = voyagerAll[5];
				p1 = voyagerAll[6];
				p2 = voyagerAll[7];
			}
			// Si pas de solution pour voyager(p20b)
			else if((solutionPa == 1) && (solutionPb == 0)){
			    // dans le repre tournant
				q1 = voyagerAll[0];
				q2 = voyagerAll[1];
				p1 = voyagerAll[2];
				p2 = voyagerAll[3];
			}
			// sinon on prend la solution avec p20a
			else{
			    // dans le repre tournant
				q1 = voyagerAll[0];
				q2 = voyagerAll[1];
				p1 = voyagerAll[2];
				p2 = voyagerAll[3];
			}

			
			appel = (int) voyagerAll[8];
		}
		else if(solutionP == 0){
			ptsTrajP = 2*nbPtsTrajP;
		}
		//-----------------------------------------------------------
		
		// dans le repere fixe
		q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
		q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);
		
		// calcul de l'integrale de jacobi
		voyager[0] = q1;
		voyager[1] = q2;
		voyager[2] = p1;
		voyager[3] = p2;
		jacobi = mouvement.intPrem(voyager,mutab);
		jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
	}
	
	else if(ptsTrajP >= nbPtsTrajP){
		double[] retourSolP = new double[3];
		
		//-----------------------------------------------------------
		// Changement des valeurs initiales pour aller sur une autre trajectoire
		//-----------------------------------------------------------
		// on incrmente q10 si infrieur  q10max.
		if(q10 >= q10max){
			compteurq10++;
			q10 = q10min;
		}
		else{
			q10 = q10 + pasq10;
		}
		// incrmente p10 et/ou q10 si p10 = p10max, et/ou si q10 = q10max 
		// on incrmente p10 si q10 a atteint q10max
		if(p10 >= p10max){
			return null;
		}
		else if(p10 < p10max){
			p10 = p10min + compteurq10*pasp10;
		}
		
		retourSolP = sectionDePoincare.calculp2H0(q10, q2Initial, p10, mu);
		p20a = retourSolP[0];
		p20b = retourSolP[1];
		solutionP = (int) retourSolP[2];
		
		// Tant qu'il n'y a pas de solution pour la varit d'nergie choisie 
		while (solutionP == 0){
			// on incrmente q10 si infrieut  q10max.
			if(q10 >= q10max){
				compteurq10++;
				q10 = q10min;
			}
			else{
				q10 = q10 + pasq10;
			}
			// incrmente p10 et/ou q10 si p10 = p10max, et/ou si q10 = q10max 
			// on incrmente p10 si q10 a atteint q10max
			if(p10 >= p10max){
				return null;
			}
			else if(p10 < p10max){
				p10 = p10min + compteurq10*pasp10;
			}
			
			retourSolP = sectionDePoincare.calculp2H0(q10, q2Initial, p10, mu);
			p20a = retourSolP[0];
			p20b = retourSolP[1];
			solutionP = (int) retourSolP[2];
		}
		
		ptsTrajP = 0;		// rinitialisation, car une solution est trouve 
							// pour p20, tant donns les nouveaux p10 et q10
		//-----------------------------------------------------------
		
		//-----------------------------------------------------------
		// Reinitialisation des variables courantes
		//-----------------------------------------------------------
		
		q1 = voyager[0];
		q2 = voyager[1];
		p1 = voyager[2];
		p2 = voyager[3];
		
		// On rinitialise avec ces nouvelles CI
		voyagerAll[0] = q10;
		voyagerAll[1] = q2Initial;
		voyagerAll[2] = p10;
		voyagerAll[3] = p20a;
		voyagerAll[4] = q10;
		voyagerAll[5] = q2Initial;
		voyagerAll[6] = p10;
		voyagerAll[7] = p20b;
		//-----------------------------------------------------------

		//-----------------------------------------------------------
		// On intgre pour aller au prochain point de la section car 
		// sinon trace un point n'appartenant pas  la section
		//-----------------------------------------------------------
		voyagerAll = sectionDePoincare.calculVoyager(voyagerAll, mu, fEqMvmt, deltaT, precision, surfaceSection, appelP);
		solutionP = (int) voyagerAll[9];
		solutionPa = (int) voyagerAll[10];
		solutionPb = (int) voyagerAll[11];
		
		if (solutionP == 1){
			// Compteur de points tracs
			ptsTrajP ++;
			// Si pas de solution pour voyager(p20a)	
			if((solutionPa == 0) && (solutionPb == 1)){
			    // dans le repre tournant
				q1 = voyagerAll[4];
				q2 = voyagerAll[5];
				p1 = voyagerAll[6];
				p2 = voyagerAll[7];
			}
			// Si pas de solution pour voyager(p20b)
			else if((solutionPa == 1) && (solutionPb == 0)){
			    // dans le repre tournant
				q1 = voyagerAll[0];
				q2 = voyagerAll[1];
				p1 = voyagerAll[2];
				p2 = voyagerAll[3];
			}
			// sinon on prend la solution avec p20a
			else{
			    // dans le repre tournant
				q1 = voyagerAll[0];
				q2 = voyagerAll[1];
				p1 = voyagerAll[2];
				p2 = voyagerAll[3];
			}

			
			appel = (int) voyagerAll[8];
		}
		else if(solutionP == 0){
			ptsTrajP = 2*nbPtsTrajP;
		}
		//-----------------------------------------------------------
		
		// dans le repere fixe
		q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
		q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);
		
		// calcul de l'integrale de jacobi
		voyager[0] = q1;
		voyager[1] = q2;
		voyager[2] = p1;
		voyager[3] = p2;
		jacobi = mouvement.intPrem(voyager,mutab);
		//System.out.println("J = " + jacobi + "  CHGT l'appel = " + appel);
		// ----------- DEBUGING ------------
		/*if((Math.abs(jacobi)>1.001*jacobiP)||(Math.abs(jacobi)<0.999*jacobiP)){
			System.out.println("-------------------------");
			System.out.println("J = " + jacobi + "  CHGT l'appel = " + appel);
			System.out.println("soluce = " + solutionP + " ptsTraj = " + ptsTrajP);
			System.out.println("q1 = " + q1 + " ; q2 = " + q2 + " ; p1 = " + p1 + " ; p2 = " + p2);
		}*/
		jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
	}
	
	else{
		//-----------------------------------------------------------
		// Sauvegarde des variables, au cas o il n'y ait plus de solution 
		// sur cette trajectoire
		//-----------------------------------------------------------
		q1 = voyager[0];
		q2 = voyager[1];
		p1 = voyager[2];
		p2 = voyager[3];
		//-----------------------------------------------------------
		
		//-----------------------------------------------------------
		// Intgration des quations pour trouver le prochain point de la section
		//-----------------------------------------------------------
		voyagerAll = sectionDePoincare.calculVoyager(voyagerAll, mu, fEqMvmt, deltaT, precision, surfaceSection, appelP);
		solutionP = (int) voyagerAll[9];
		solutionPa = (int) voyagerAll[10];
		solutionPb = (int) voyagerAll[11];
		
		if (solutionP == 1){
			// Compteur de points tracs
			ptsTrajP ++;
			// Si pas de solution pour voyager(p20a)	
			if((solutionPa == 0) && (solutionPb == 1)){
			    // dans le repre tournant
				q1 = voyagerAll[4];
				q2 = voyagerAll[5];
				p1 = voyagerAll[6];
				p2 = voyagerAll[7];
			}
			// Si pas de solution pour voyager(p20b)
			else if((solutionPa == 1) && (solutionPb == 0)){
			    // dans le repre tournant
				q1 = voyagerAll[0];
				q2 = voyagerAll[1];
				p1 = voyagerAll[2];
				p2 = voyagerAll[3];
			}
			// sinon on prend la solution avec p20a
			else{
			    // dans le repre tournant
				q1 = voyagerAll[0];
				q2 = voyagerAll[1];
				p1 = voyagerAll[2];
				p2 = voyagerAll[3];
			}

			
			appel = (int) voyagerAll[8];
		}
		else if(solutionP == 0){
			ptsTrajP = 2*nbPtsTrajP;
		}
		//-----------------------------------------------------------
		
		/* ------- DEBUGING ------------
		if(Math.abs(q2)>1.e-2){
			System.out.println("BUG");
		}*/
		
		// dans le repere fixe
		q1fixe = q1*Math.cos(appel*deltaT) - q2*Math.sin(appel*deltaT);
		q2fixe = q1*Math.sin(appel*deltaT) + q2*Math.cos(appel*deltaT);
		
		// calcul de l'integrale de jacobi
		voyager[0] = q1;
		voyager[1] = q2;
		voyager[2] = p1;
		voyager[3] = p2;
		jacobi = mouvement.intPrem(voyager,mutab);
		// ----------- DEBUGING ------------
		//System.out.println("Jacobi = " + jacobi);
		//System.out.println("Jacobi0 = " + jacobiInitial);
		//System.out.println("JacobiP = " + jacobiP);
		//System.out.println("J = " + jacobiP + "  l'appel = " + appel);
		/*if((Math.abs(jacobi)>1.001*jacobiP)||(Math.abs(jacobi)<0.999*jacobiP)){
			System.out.println("-------------------------");
			System.out.println("J = " + jacobi + "  l'appel = " + appel);
			System.out.println("soluce = " + solutionP + " ptsTraj = " + ptsTrajP);
			System.out.println("q1 = " + q1 + " ; q2 = " + q2 + " ; p1 = " + p1 + " ; p2 = " + p2);
		}*/
		jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
	}
	
    // remplissage du tableau de parametres de sortie
	// ----------------------------------------------
    // pour la position des courbes de vitesse nulles
    out.ajouterDouble("q1", q1);
    out.ajouterDouble("q2", q2);
    out.ajouterDouble("q11", q11);
    out.ajouterDouble("q21", q21);
    out.ajouterDouble("q12", q12);
    out.ajouterDouble("q22", q22);
    // pour l'integrale de Jacobi
    out.ajouterDouble("jacobiRelatif", jacobiRelatif);
    out.ajouterDouble("appel", appel);
    // pour la position de la sonde dans le repere fixe
    out.ajouterDouble("q1fixe", q1fixe);
    out.ajouterDouble("q2fixe", q2fixe);
    out.ajouterDouble("p1", p1);
    out.ajouterDouble("p2", p2);
    // pour avoir une ide de la valeur initiale de l'integrale de Jacobi
    //out.ajouterString("jacobiInitial", "valeur initiale de l'intgrale de Jacobi " + jacobiInitial);
    //out.ajouterString("jacobiInitial", "appel = " + appel + " | J = " + jacobi);
    out.ajouterString("jacobiInitial", "p10 = " + p10 + " | p10min = " + p10min + " | p10max = " + p10max);
    
	return out;

}

else {
    return null;
    }
}
}

