package eqmvt;
import java.lang.Math;

/**
 * Classe test qui implmente l'quation d'un oscillateur harmonique  2D.
 * @author Mathieu HAVEL, Pierre INIZAN
 * @version 12/01/2007
 * @since Projet Trois Corps 2007
 * @see TestIntegrateur
 **/



public class EqMvmtTestIntegrateur implements InterfaceEqMvmt{

  /**
   * quation du mouvement
   * @param  x            Tableau de la forme [Q1, Q2, P1, P2]
   * @param  parametres   Tableau de paramtres (masse, k1, k2)
   * @return Tableau de la forme [f1(X), f2(X)] = [Q1_dot, Q2_dot, P1_dot, P2_dot] 
   */

  public double[] eqMvmt(double[] x, double[] parametres){

    double[] xdot=new double[x.length];

    xdot[0] = x[2]/parametres[0];
    xdot[1] = x[3]/parametres[0];
    xdot[2] = -parametres[1]*x[0];
    xdot[3] = -parametres[2]*x[1];

    return xdot;

  }



  /**
   * Matrice jacobienne associe  l'quation du mouvement
   * @param  x            Tableau de la forme [Q1, Q2, P1, P2]
   * @param  parametres   Tableau de paramtres
   * @return   Tableau de la forme [df1/dx1, df2/dx1, df1/dx2, ...] (ATTENTION : remplissage colonne par colonne)    
   */

  public double[] jacMvmt(double[] x, double[] parametres){

    double[] Df = new double[x.length * x.length];
    Df[0] = 0;
    Df[1] = 0;
    Df[2] = -parametres[1];
    Df[3] = 0;

    Df[4] = 0;
    Df[5] = 0;
    Df[6] = 0;
    Df[7] = -parametres[2];

    Df[8] = 1/parametres[0];
    Df[9] = 0;
    Df[10] = 0;
    Df[11] = 0;
    
    Df[12] = 0;
    Df[13] = 1/parametres[0];
    Df[14] = 0;
    Df[15] = 0;

    return Df;
  }



  /**
   * Implmente une intgrale premire du mouvement
   * @param  x            Tableau de la forme [Q1, Q2, P1, P2]
   * @param  parametres   Tableau de paramtres
   * @return              Valeur de l'intgrale premire
   */

  public double intPrem(double[] x, double[] parametres){

    double jacobi=-( (Math.pow(x[2],2) + Math.pow(x[3],2))/parametres[0] + Math.pow(x[0],2)*parametres[1] + Math.pow(x[1],2)*parametres[2] );
    return jacobi;  

  }

}
