package eqmvt;
/**
 * Classe patron pour toute fonction implmentant une quation du mouvement
 * et/ou la matrice jacobienne associe  cette quation  
 * et/ou une intgrale premire du mouvement
 *
 * @author Nicolas DELSATE (version du 17/01/2006)
 * <p>Mathieu HAVEL, Pierre INIZAN (version du 19/01/2007)
 * @version 19/01/2007
 * @since Projet Trois Corps 2006
 */

public interface InterfaceEqMvmt{

  /**
   * Mthode de calcul d'quation du mouvement a dfinir dans une classe hrite de celle-ci.
   * @param  x            Tableau de la forme [X1,X2,...]
   * @param  parametres   Tableau de paramtres
   * @return   Tableau de la forme [X1_dot,X2_dot,...]
   * @since Projet Trois Corps 2006
   */
  public double[] eqMvmt(double[] x,double[] parametres);
  
  
  /**
   * Matrice jacobienne Df de l'quation f du mouvement (x_dot = f(x)), f : R^n -> R^p
   * @param  x  Tableau de la forme [X1,X2,...]
   * @param  parametres   Tableau de parametres
   * @return  Tableau de la forme [df1/dx1, df2/dx1, ..., dfp/dx1, df1/dx2, ..., dfp/dxn]
   * @since Projet Trois Corps 2006
   */
   
  public double[] jacMvmt(double[] x, double[] parametres);  
  

  /**
   * Implmente une intgrale premire
   * @param  x            Tableau de la forme [X1,X2,...]
   * @param  parametres   Tableau de paramtres
   * @return              Valeur de l'integrale premire
   * @since Projet Trois Corps 2006
   */
    public double intPrem(double[] x,double[] parametres);
  
  

}
