package fonction;
import eqmvt.*;

/**
 * Classe qui implmente une fonction g hrite de ChoixDeFonction  partir d'une fonction f hrite de InterfaceEqMvmt, telle que g(x) = cste_add + cste_mult*f(x) - x
 * f, cste_add et cste_mult sont dfinies dans le constructeur.
 * <p>
 * Utilisation dans l'intgrateur symplectique (rk4sympf, dans Integrateur.java), o f reprsente l'quation du mouvement, donne dans le constructeur de la classe Integrateur,  et g la fonction  annuler par l'algorithme de Newton. Celui-ci prend en entre la fonction g et sa matrice jacobienne Dg. La classe JacNewtSymp implmente cette matrice.
 * @see eqmvt.InterfaceEqMvmt InterfaceEqMvmt 
 * @see ChoixDeFonction
 * @see JacNewtSymp
 * @see mn.Integrateur Integrateur
 * @see mn.DetermineZero DetermineZero
 *
 * @author Mathieu HAVEL, Pierre INIZAN
 * @version 07/01/2007
 * @since Projet Trois Corps 2007
 */



public class FctNewtSymp implements ChoixDeFonction {


    private InterfaceEqMvmt f;
    private double[] cste_add;
    private double cste_mult;

    //Constructeur
    /**
     * Constructeur
     * @param f Equation du mouvement
     * @param cste_add Facteur additif intervenant dans la dfinition des ksi de l'intgrateur RK4 symp
     * @param cste_mult Facteur multiplicatif intervenant dans la dfinition des ksi de l'intgrateur RK4 symp
     */
    public FctNewtSymp(InterfaceEqMvmt f, double[] cste_add, double cste_mult){

  /**
   * @param f fonction hrite de InterfaceEqMvmt  partir de laquelle on va crer la nouvelle fonction
   * @param cste_add constante additive qui intervient dans la construction de la fonction
   * @param cste_mult constante multiplicative qui intervient dans la construction de la fonction
   * @return une fonction g hrite de ChoixDeFonction telle que g(x) = cste_adde + cste_multe*f(x) - x
   */

  this.f = f;
  this.cste_add = cste_add;
  this.cste_mult = cste_mult;
    }    
    
    //Mthode vide mais ncessaire pour que FctNewtSymp hrite de ChoixDeFonction
    public double fonction(double variable, double[] parametres) {
  double resultat = 0;
  return resultat;
    }
    
    //Mthode vide mais ncessaire pour que FctNewtSymp hrite de ChoixDeFonction
    public double fonction(double[] variable, double[] parametres) {
  double resultat = 0;
  return resultat;
    }



  /**
   * @param variable Position en laquelle on value la fonction
   * @param parametres Paramtres utiliss par la fonction f
   * @param taille Dimension de l'espace d'arrive de la fonction
   * @return Valeur de la fonction value au point "variable"
   */

    //Dfinition de la nouvelle fonction g, dfinie  partir de f    
    public double[] fonction_multi(double[] variable, double[] parametres, int taille) {

  int n = variable.length;
  int i;
  double[] resultat = new double[taille];
  double[] ff;
  ff = f.eqMvmt(variable,parametres); //fonction de base, situee dans la methode eqMvmt, value au point "variable"
  for(i=0;i<n;i++){
      resultat[i] = cste_mult*ff[i] - variable[i] + cste_add[i];
  }  
  return resultat; //renvoie g(variable)
    }
    
}
