package mn;
import eqmvt.*;
import fonction.*;
import java.lang.Math;

//NB : compilation pour javadoc:    javadoc -d Documentation Integrateur.java

/**
 * Classe contenant un ensemble de mthodes pour intgrer un problme donn par l'quation de mouvement "eqMvmt()" (qui est implmente dans une classe qui implmente UsineEqMvmt.java). On utilise une mthode runge-kutta d'ordre 4 "rk4()", runge-kutta  pas variable "rk4pv()", et une mthode symplectique d'ordre 4 "rk4sympf()".
 * <ul>
 * <li><b> h </b>         <dd>Pas d'integration courant initialis  0 dans le constructeur</dd>
 * <li><b>taille </b>      <dd>Taille du tableau que l'on passe en paramtre du constructeur, quivalent au nombre d'quation du mouvement</dd>
 * <li><b>parametres </b>  <dd>Tableau de paramtres pass  la methode "eqMvmt()"</dd>
 * <li><b>nbrParam </b>    <dd>Taille du tableau paramtres</dd>
 * <li><b>xNew </b>        <dd>Tableau servant  l'adaptation du pas, utilis dans rk4pv</dd>
 * <li><b>tNew </b>        <dd>Temps au dernier point trouv par l'algo, de maniere gnrale = tNew+h ; au debut il est = 0</dd>
 * <li><b>tFinal </b>      <dd>Temps auquel on demande une rponse, il sera de manire gnrale = tFinal+intTps ; au debut il est = 0</dd>
 * <li><b>erreur </b>      <dd>Valeur de la dernire erreur commise : sert pour l'adaptation du pas</dd>
 * <li><b>mvmt </b>       <dd>Contient le "nom" de l'quation du mouvement; on aura accs  l'quation en faisant mvmt.eqMvmt</dd>
 * </ul>
 * @see eqmvt.InterfaceEqMvmt
 *
 * @author Nicolas DELSATE (version du 17/01/2006)
 * <p>Mathieu HAVEL, Pierre INIZAN, Guillaume BOUBIN, Gaetan LE CHAT version du 10/01/2007)
 * @version 10/01/2007
 * @since Projet Trois Corps 2006
 *
 */

public class Integrateur{

 /**
  * <ul>
  * <li><b> h </b>         <dd>Pas d'intgration courant initialise  0 dans le constructeur</dd>
  * <li><b> taille </b>    <dd>Taille du tableau que l'on passe en paramtre du constructeur, quivalent au nombre d'quations du mouvement</dd>
  * <li><b> parametres </b><dd>Tableau de paramtres passe  la mthode "eqMvmt()"</dd>
  * <li><b> nbrParam </b>  <dd>Taille du tableau paramtres  </dd>        
  * <li><b> xNew </b>      <dd>Tableau servant  l'adaptation du pas, utilise dans rk4pv</dd>
  * <li><b> tNew </b>      <dd>Temps au dernier pointt trouve par l'algo, de manire gnrale = tNew+h ; au debut il est = 0</dd>
  * <li><b> tFinal </b>    <dd>Temps auquel on demande une rponse, il sera de manire gnrale = tFinal+intTps ; au debut il est = 0</dd>
  * <li><b> erreur </b>    <dd>Valeur de la dernire erreur commise : sert pour l'adaptation du pas</dd>
  * <li><b> mvmt </b>      <dd>Contient le "nom" de l'quation du mouvement; on fera accs  l'quation en faisant mvmt.eqMvmt</dd>
  * </ul>
  */
   
  double h;//le pas utilise dans l'integration courante, initialement =0


  int taille; //taille du tableau que l'on passe en parametre du constructeur, equivalent au nombre d'equation du mouvement


  double[] parametres; //tableau de parametres
  int nbrParam; //nombre de parametres


  //Le tableau et le tps suivant servent a l'adaptation du pas
  //variables utilisees dans rk4pv !!!
  double[] xNew;

  //tps ou on calcule le tableau ci-dessus
  double tNew; //tps au dernier pt trouve par l'algo, de maniere generale = tNew+h ; au debut il est = 0

  //tps auquel on demande 1reponse, il sera de maniere generale = tFinal+intTps ; au debut il est = 0
  double tFinal;

  //valeur de la derniere erreur commise : sert pour l'adaptation du pas
  double erreur;

  InterfaceEqMvmt mvmt;//contient le "nom" de l'equation du mouvement; on fera acces a l'equation en faisant mvmt.eqMvmt

 
 


  // constructeur de la classe
  /**
   *@param nbrVariable Nombre d'quation du mouvement (ou de manire nombre de variables)
   *@param tabParametres Tableau contenant les diffrents paramtres
   *@param fEqMvmt Nom du Fichier contenant la mthode qui implmente la mthode eqMvmt de l'interface InterfaceEqMvmt
   */
  public Integrateur(int nbrVariable,double[] tabParametres,String fEqMvmt) {
    taille=nbrVariable;//taille du tableau

    nbrParam=tabParametres.length;//taille du tableau de parametre
    parametres=new double[nbrParam]; //allocation de memoire pr tableau de parametres
    for(int i=0;i<nbrParam;i++) {
      parametres[i]=tabParametres[i];//tableau de parametre
    }

    //initialisation d'un pas nul
    h=0;

    //creation de mvmt de type InterfaceEqMvmt
    mvmt=UsineEqMvmt.nouveau(fEqMvmt);

    //initialisation des variables utilisees dans rk4pv !!!
    xNew=new double[taille];
    tNew=0;
    tFinal=0;
  }






  //////////////////////////////////////////////////////////////////
  //                          rk4                                 //
  //////////////////////////////////////////////////////////////////

  /**  
   * Mthode qui utilise une mthode Runge-Kutta  pas fixe d'ordre 4
   * pour calculer  partir d'un vecteur x et d'un pas une solution xPlush
   * <p>Formule gnrale : x(t+h)=x(t)+h/6*(k1+2*k2+2*k3+k4);
   * @param  x  Tableau de la forme [X1,X2,...]
   * @param  hh Pas d'intgration
   * @return    Un tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4(double[] x,double hh){
    //formule generale : x(t+h)=x(t)+h/6*(k1+2*k2+2*k3+k4);

    //les 4 facteurs utilises dans la methode runge-kutta
    double[] k1=new double[taille];
    double[] k2=new double[taille];
    double[] k3=new double[taille];
    double[] k4=new double[taille];

    //calcul du facteur k1
    k1=mvmt.eqMvmt(x,parametres);

    //tabeau transit utilise pour calculer les autres facteurs
    double[] transit=new double[taille];

    //calcul du facteur k2
    for(int i=0; i<taille; i++){
      transit[i]=x[i]+hh/2*k1[i];
    }
    k2=mvmt.eqMvmt(transit,parametres);


    //calcul du facteur k3
    for(int i=0; i<taille; i++){
      transit[i]=x[i]+hh/2*k2[i];
    }
    k3=mvmt.eqMvmt(transit,parametres);

    //calcul du facteur k4
    for(int i=0; i<taille; i++){
      transit[i]=x[i]+hh*k3[i];
    }
    k4=mvmt.eqMvmt(transit,parametres);

    
    //la reponse x(t+hh)
    double[] xPlush=new double[taille];
    for(int i=0;i<taille;i++){
      xPlush[i]=x[i]+hh/6*(k1[i]+2*k2[i]+2*k3[i]+k4[i]);
    }

    return xPlush;
  }




  //////////////////////////////////////////////////////////////////
  //                      rk4sympf                      //        
  //////////////////////////////////////////////////////////////////


  //Auteurs : Mathieu Havel et Pierre Inizan
  //cree le 05/12/06
  //modifie le 07/01/07


  /**
   * Runge-Kutta symplectique d'ordre 4 pas fixe pour passer de x(t)  x(t+h) :
   * <p>x(t+h) = x(t) + h*b1*f(t+b1/2*h,e1) + h*b2*f(t+(b1+b2/2)*h,e2) + h*b3*f(t+(b1+b2+b3/2)*h,e3)
   * avec :
   * <p>e1 = x(t) + h*b1/2*f(t+b1/2*h,e1)
   * <p>e2 = x(t) + h*b1*f(t+(b1+b2/2)*h,e1) + h*b2/2*f(t+(b1+b2/2)*h,e2)
   * <p>e3 = x(t) + h*b1*f(t+(b1+b2+b3/2)*h,e1) + h*b2*f(t+(b1+b2+b3/2)*h,e2) + h*b3/2*f(t+(b1+b2+b3/2)*h,e3)
   * <p>e1, e2, e3 sont dtermins successivement, par un algorithme de Newton (multidimensionnel)
   * @param x Tableau contenant la position initiale x(t)
   * @param hh Pas d'intgration
   * @param eps Prcision souhaite
   * @param parametre Paramtre de relaxation pour l'algorithme de Newton
   * @return Nouvelle position aprs intgration x(t+h)
   * @see DetermineZero
   * @see fonction.FctNewtSymp
   * @see fonction.JacNewtSymp
   * @since Projet Trois Corps 2007
   */

  public double[] rk4sympf(double[] x, double hh, double eps, double relax){

   boolean debugging = false;
   int i;

   // valeurs des paramtres b1, b2 et b3 de l'algorithme
   double b1 = 1./3.*(2. + Math.pow(2, 1./3.) + Math.pow(2, -1./3.));
   double b3 = b1;
   double b2 = 1 - 2*b3;

   //les 3 facteurs utilises dans la methode runge-kutta
   double[] e1=new double[taille];
   double[] e2=new double[taille];
   double[] e3=new double[taille];

   //Utilisation de l'algorithme de Newton pour dterminer les trois coefficients ei

   DetermineZero algo = new DetermineZero(); 

   double[] cste_add1 = new double[taille];
   for (i=0;i<taille;i++){
      cste_add1[i] = x[i]; 
   }

   FctNewtSymp g1 = new FctNewtSymp(mvmt, cste_add1, hh*b1/2); // dfinition de la fonction g1 : x -> cste_add1 + hh*b1/2*f(x) - x
   JacNewtSymp Dg1 = new JacNewtSymp(mvmt, hh*b1/2); // dfinition de la matrice jacobienne de g1, Dg1 : x -> hh*b1/2*Df(x) - Id

   // Calcul de e1 par algorithme de Newton : g1(e1) = 0
   e1 = algo.newton_multidim(taille, eps, relax, cste_add1, g1, parametres, Dg1, parametres); // parametres = mu : intervient pour l'appel des fonctions g1 et Dg1

   if(debugging){
      System.out.println("e1[0] ="+e1[0]);
      System.out.println("e1[1] ="+e1[1]);
      System.out.println("e1[2] ="+e1[2]);
      System.out.println("e1[3] ="+e1[3]);
   }

   double[] cste_add2 = new double[taille];
   double[] tmp1 = mvmt.eqMvmt(e1, parametres);

   for (i=0;i<taille;i++){
       cste_add2[i] = cste_add1[i] + hh*b1*tmp1[i];  
    }

    FctNewtSymp g2 = new FctNewtSymp(mvmt, cste_add2, hh*b2/2); // dfinition de la fonction g2 : x -> cste_add2 + hh*b2/2*f(x) - x
    JacNewtSymp Dg2 = new JacNewtSymp(mvmt, hh*b2/2); // dfinition de la matrice jacobienne de g2, Dg2 : x -> hh*b2/2*Df(x) - Id


    // Calcul de e2 par algorithme de Newton : g2(e2) = 0  
    e2 = algo.newton_multidim(taille, eps, relax, cste_add2, g2, parametres, Dg2, parametres); 

    if(debugging){
       System.out.println("e2[0] ="+e2[0]);
       System.out.println("e2[1] ="+e2[1]);
       System.out.println("e2[2] ="+e2[2]);
       System.out.println("e2[3] ="+e2[3]);
    }

    double[] cste_add3 = new double[taille];
    double[] tmp2 = mvmt.eqMvmt(e2, parametres);
    for (i=0;i<taille;i++){
        cste_add3[i] = cste_add2[i] + hh*b2*tmp2[i];  
    }

    FctNewtSymp g3 = new FctNewtSymp(mvmt, cste_add3, hh*b3/2); // g3 : x -> cste_add3 + hh*b3/2*f(x) - x
    JacNewtSymp Dg3 = new JacNewtSymp(mvmt, hh*b3/2); // Dg3 : x -> hh*b3/2*Df(x) - Id

    // Calcul de e3 par algorithme de Newton : g3(e3) = 0  
    e3 = algo.newton_multidim(taille, eps, relax, cste_add3, g3, parametres, Dg3, parametres); // parametres = mu

    if(debugging){
       System.out.println("e3[0] ="+e3[0]);
       System.out.println("e3[1] ="+e3[1]);
       System.out.println("e3[2] ="+e3[2]);
       System.out.println("e3[3] ="+e3[3]);
    }

    //la reponse x(t+hh)
    double[] xPlush=new double[taille];
    double[] tmp3 = mvmt.eqMvmt(e3, parametres);
    for(i=0;i<taille;i++){
      xPlush[i]=x[i]+hh*(b1*tmp1[i] + b2*tmp2[i] + b3*tmp3[i]);
    }

    if(debugging){
       System.out.println("RK4 : pos[0] = " +xPlush[0]);
       System.out.println("RK4 : pos[1] = " +xPlush[1]);
       System.out.println("RK4 : pos[2] = " +xPlush[2]);
       System.out.println("RK4 : pos[3] = " +xPlush[3]);
    }

    return xPlush; //renvoie x(t+hh)

  }

  //////////////////////////////////////////////////////////////////
  //                          rk4pv                               //
  //////////////////////////////////////////////////////////////////

  /**
   * Runge-Kutta d'ordre 4  pas variable avec une precision  prciser
   * Cette mthode adapte le pas lors de son intgration mais aussi adapte le pas pour que la dernire tape donne exactement le rsultat au temps "intTps" voulu sans faire d'interpolation !
   * @param  x           Tableau de la forme [X1,X2,...]
   * @param  intTps      Interval de temps durant lequel on fait l'intgration, c'est aprs ce temps qu'on veut une rponse. 
   * @param  precision   Prcision sur la diffrence relative entre deux resultats. Si l'on atteint cette prcision, on renvoit la solution. Sinon, on diminue le pas et on recommence
   * @param  vieuxCalcul =FALSE si on veut faire un nouveau calcul et =TRUE si on continue un ancien calcul; dans ce cas, on ne tient pas compte du "x" passe en argument
   * @return             Tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double[] x,double intTps,double precision, boolean vieuxCalcul){

    //test preliminaire
    if(intTps<=0) {
      System.out.println("!?!  intTps doit etre >0   !?!");
    }


    //creation de variables propre a la methode
    //x(t+h)
    double[] x1Plush=new double[taille];

    //x(t+h/2)
    double[] x2aPlushDemi=new double[taille];

    //x(t+h) obtenu a partir de x(t+h/2)
    double[] x2Plush=new double[taille];

    //erreur relative commise lors de l'integration
    double[] delta = new double[taille]; //contiendra |x1Plush-x2Plush|/(|x1Plush|+|x2Plush|+x') i.e l'erreur relative commise lors de l'integration 


    //initialisation du pas maximum h
    double hmax=intTps;


    //mise a jour de la variable GLOBALE tFinal
    //tps a l'interpolation ou l'on demande la solution
    tFinal=tFinal+intTps;



    //si le pas h est nul, faut bien lui donner qq chose
    //En fait, lorsqu'on appel la fonction la premiere fois, le pas h est 
    //egale a 0. Il faut donc l'initialiser a qq chose.
    //On donne l'interval de temps d'integration (hmax), donc, on initialise 
    //ce pas h avec ce intTps.


    if(h==0){
      h=hmax;
    }

    //sinon, on ne tient pas compte de intTps pour determiner h, on utilise le 
    //pas h qui est declare en global car on a deja appele la methode 
    //precedemment.


    if(! vieuxCalcul){ //c'est un nouveau calcul

      //initialisation de xNew[],variable utilisee ds ts les calculs suivants
      for(int i=0;i<taille;i++){
        xNew[i]=x[i];
      }

      //initialisation de l'erreur pour pouvoir, lors du premier passage 
      //dans la boucle do{...}while(erreur>precision), commencer avec un pas 
      //qui vaut h (en effet, cette boucle commence par une adaptation du 
      //pas !
      erreur=precision*Math.pow(0.9,5);
    }
    //else on est dans la suite d'un calcul deja commence donc, ne tient pas 
    //compte de x donne en parametre et on redemarre avec l'ancien xNew


    //pour pouvoir rentrer dans la boucle qui suit
    boolean dernier=false;
  
    while(! dernier) { //"presque" equivalent a tNew<tFinal)

      //recherche du pas optimal pour calculer le xNew
      do { 
        dernier=false;
 
        //adaptation du pas : - augmente si on entre dans la boucle car 
        //                      erreur<precision. A cause du facteur 
        //                      (de securite) O.9, il y a une augmentation 
        //                      lorsque erreur<1.6935088
        //                    - diminue si on viens de la fin de la boucle car 
        //                      erreur>precision
        h=h*0.9*Math.pow(precision/erreur,0.2);


        //si le pas calcule est plus grand que le pas maximum autorise
        if(h>hmax) {
          h=hmax;
        //---Debugging----
        //System.out.println("-----------h>hmax => h=hmax=" + h);
        //---Debugging----
      }
  
      //si le pas h calcule est tel qu'avec celui-ci, on donnera une 
      //solution a un temps plus grand que celui demande, alors, on adapte le
      //pas en l'egalant au pas qu'il faudrait en partant de tNew pour 
      //pouvoir finir au temps exacte demande (tFinal).
      if(h> tFinal-tNew ) {
        h=tFinal-tNew;

        dernier=true;//puisqu'on adapte le pas de cette facon, c'est 
                     //qu'apres, il n'y aura plus d'autre pas.
      }

      //calcul de x(t+h)
      x1Plush=rk4(xNew,h);

      //calcul de x(t+h/2)
      x2aPlushDemi=rk4(xNew,h/2);

      //calcul de x(t+h/2+h/2)
      x2Plush=rk4(x2aPlushDemi,h/2);


      //pour que l'erreur ne devienne pas infinie lors du passage d'un cote
      // a l'autre de l'axe des x, on ajoutera yPrime*h au denominateur
      double[] yPrime = new double[taille];
      yPrime = mvmt.eqMvmt(x2Plush,parametres);


      //calcul de l'erreur commise
      for(int i=0;i<taille;i++){
        delta[i]=Math.abs( x2Plush[i]-x1Plush[i] ) / ( Math.abs(x2Plush[i]) + Math.abs(x1Plush[i]) + Math.abs(yPrime[i]*h));
      }

      erreur=Math.abs(maximum(delta));



    } while(erreur>precision);

    //arriver ici, on a un pas optimal h et on met enfin a jour notre xNew et
    //tNew

    //actualiastion de xNew
    for(int i=0;i<taille;i++){
      xNew[i]=x2Plush[i] + (x2Plush[i]-x1Plush[i])/15;
      //xNew a alors une erreur d'ordre 6 au lieu de l'ordre 5
    }

    //acutalisation de tNew
    tNew=tNew+h;
  

    }//fin de while(tNew<tFinal)


    double[] xResultatT=new double[taille];

    for(int i=0; i<taille ; i++) {
      xResultatT[i]=xNew[i];
    }

    return xResultatT;
  }



  /**
   * Runge-Kutta d'ordre 4  pas variable avec une prcision prdfinie=1.0e-3
   * @param  x      Tableau de la forme [Q1,Q2,P1,P2]
   * @param  intTps Interval de temps durant lequel on fait l'intgration, c'est aprs ce temps qu'on veut une rponse
   * @return        Tableau de la forme [Q1,Q2,P1,P2]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double[] x,double intTps){
    double precision=1.0e-6;
    double[] xResultatT=new double[taille];
    xResultatT=rk4pv(x,intTps,precision);
    return xResultatT;
  }



  /**
   * Appelle la fonction rk4pv(double[] x, double deltaT, double precision, boolean vieuxCalcul) avec une prcision fixe  1.0e-6
   * @param  x           Tableau de la forme [X1,X2,...]
   * @param  deltaT      Interval de temps durant lequel on fait l'intgration, c'est aprs ce temps qu'on veut une rponse. 
   * @param  vieuxCalcul =FALSE si on veut faire un nouveau calcul et =TRUE si on continue un ancien calcul; dans ce cas, on ne tient pas compte du "x" passe en argument
   * @return             Tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double[] x,double deltaT, boolean vieuxCalcul){
    double[] xResultatT=new double[taille]; //tableau de resultat au temps deltaT exactement !!!
    double precision=1.0e-6;   
    xResultatT=rk4pv(x,deltaT,precision,vieuxCalcul);
    return xResultatT;
  }



  /**
   * Appelle la fonction rk4pv(double deltaT, double precision) avec une prcision fixe  1.0e-6
   * @param  deltaT      Interval de temps durant lequel on fait l'intgration, c'est aprs ce temps qu'on veut une rponse. 
   * @return             Tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double deltaT){
    double precision=1.0e-6;
    double[] xResultatT=new double[taille]; //tableau de resultat au temps deltaT exactement !!!
    xResultatT=rk4pv(deltaT,precision);
    return xResultatT;
  }



  /**
   * Appelle la fonction rk4pv(double[] x, double deltaT, double precision, boolean vieuxCalcul) sachant qu'on est dans la suite d'un vieux calcul. Donc, on passe un x bidon comme premier paramtre et "true" pour le paramtre boolean
   * @param  deltaT      Interval de temps durant lequel on fait l'intgration, c'est aprs ce temps qu'on veut une rponse. 
   * @param  precision   Prcision sur la diffrence relative entre deux resultats. Si l'on atteint cette prcision, on renvoit la solution. Sinon, on diminue le pas et on recommence
   * @return             Tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double deltaT, double precision){
    double[] xResultatT=new double[taille]; //tableau de resultat au temps deltaT exactement !!!
    double[] xBidon=new double[taille];
    for(int i=0;i<taille;i++){
      xBidon[i]=0;
    }
    xResultatT=rk4pv(xBidon,deltaT,precision,true);
    return xResultatT;
  }



  /**
   * Appelle la fonction rk4pv(double[] x, double deltaT, double precision, boolean vieuxCalcul) avec "false" pour le paramtre boolean
   * @param  x           Tableau de la forme [X1,X2,...]
   * @param  deltaT      Interval de temps durant lequel on fait l'intgration, c'est aprs ce temps qu'on veut une rponse. 
   * @param  precision   Prcision sur la diffrence relative entre deux resultats. Si l'on atteint cette prcision, on renvoit la solution. Sinon, on diminue le pas et on recommence
   * @return             Tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double[] x,double deltaT, double precision){
    double[] xResultatT=new double[taille]; //tableau de resultat au temps deltaT exactement !!!
    xResultatT=rk4pv(x,deltaT,precision,false);
    return xResultatT;
  }






  //////////////////////////////////////////////////////////////////
  //                       interpolation                          //
  //////////////////////////////////////////////////////////////////

  /**
   * Interpolation par une droite
   * @param y1        Valeur de la fonction valu en t1
   * @param y2        Valeur de la fonction valu en t2
   * @param tInterpol Sachant que t est le temps auquel on veut l'interpolation, tInterpol=t-t1
   * @param tTotal    t2-t1
   * @return   L'interpolation au temps t
   * @since Projet Trois Corps 2006
   */

  public double[] interpolation(double[] y1, double[] y2, double tInterpol, double tTotal){

    double[] resultatInterpolation=new double[taille];

    for(int i=0;i<taille;i++){

     resultatInterpolation[i]=y1[i]+(y2[i]-y1[i])*(tInterpol)/(tTotal);
    }

    return resultatInterpolation;
  }






  //////////////////////////////////////////////////////////////////
  //                       maximum                                //
  //////////////////////////////////////////////////////////////////

  /**
   * Calcul de la valeur de l'lment le plus grand en valeur absolue d'un tableau
   * @param tab      Tableau de double
   * @return         Valeur de l'lment le plus grand en valeur absolue d'un tableau
   * @since Projet Trois Corps 2006
   */

  public double maximum(double[] tab){
    double maxTab=Math.abs(tab[0]);
    int longueur=tab.length;

    for(int i=1;i<longueur;i++){
      if(maxTab<Math.abs(tab[i])){
         maxTab=Math.abs(tab[i]);
      }
    }
    return maxTab;
  }



  //////////////////////////////////////////////////////////////////
  //                        rk4 general                           //
  //////////////////////////////////////////////////////////////////

  /**  
   * Mthode qui utilise une mthode Runge-Kutta  pas fixe d'ordre 4
   * pour calculer  partir d'un vecteur x et d'un pas une solution xPlush
   * <p>Formule gnrale : x(t+h)=x(t)+h/6*(k1+2*k2+2*k3+k4);
   * @param  x  Tableau de la forme [X1,X2,...]
   * @param  hh Pas d'intgration
   * @param aIntegrer Fonction  intgrer
   * @return    Un tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4(double[] x,double hh,ChoixDeFonction aIntegrer){
    //formule generale : x(t+h)=x(t)+h/6*(k1+2*k2+2*k3+k4);

    //les 4 facteurs utilises dans la methode runge-kutta
    double[] k1=new double[taille];
    double[] k2=new double[taille];
    double[] k3=new double[taille];
    double[] k4=new double[taille];

    //calcul du facteur k1
    k1=aIntegrer.fonction_multi(x, parametres, taille);

    //tableau transit utilise pour calculer les autres facteurs
    double[] transit=new double[taille];

    //calcul du facteur k2
    for(int i=0; i<taille; i++){
      transit[i]=x[i]+hh/2*k1[i];
    }
    k2=aIntegrer.fonction_multi(transit,parametres,taille);


    //calcul du facteur k3
    for(int i=0; i<taille; i++){
      transit[i]=x[i]+hh/2*k2[i];
    }
    k3=aIntegrer.fonction_multi(transit,parametres,taille);

    //calcul du facteur k4
    for(int i=0; i<taille; i++){
      transit[i]=x[i]+hh*k3[i];
    }
    k4=aIntegrer.fonction_multi(transit,parametres,taille);

    
    //la reponse x(t+hh)
    double[] xPlush=new double[taille];
    for(int i=0;i<taille;i++){
      xPlush[i]=x[i]+hh/6*(k1[i]+2*k2[i]+2*k3[i]+k4[i]);
    }

    return xPlush;
  }




  //////////////////////////////////////////////////////////////////
  //                          rk4pv general                       //
  //////////////////////////////////////////////////////////////////

  /**
   * Runge-Kutta d'ordre 4  pas variable avec une prcision  dfinir
   * Cette mthode adapte le pas lors de son intgration mais aussi adapte le pas pour que la dernire tape donne exactement le rsultat au temps "intTps" voulu sans faire d'interpolation !
   * @param  x           Tableau de la forme [X1,X2,...]
   * @param  intTps      Interval de temps durant lequel on fait l'intgration, c'est aprs ce temps qu'on veut une rponse. 
   * @param  precision   Prcision sur la diffrence relative entre deux rsultats. Si l'on atteint cette prcision, on renvoit la solution. Sinon, on diminue le pas et on recommence
   * @param  aIntegrer Fonction  intgrer
   * @param  vieuxCalcul =FALSE si on veut faire un nouveau calcul et =TRUE si on continue un ancien calcul; dans ce cas, on ne tient pas compte du "x" passe en argument
   * @return             Tableau de la forme [X1,X2,...]
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double[] x,double intTps,double precision,ChoixDeFonction aIntegrer, boolean vieuxCalcul){

    //test preliminaire
    if(intTps<=0) {
      System.out.println("!?!  intTps doit etre >0   !?!");
    }



    //creation de variables propre a la methode
    //x(t+h)
    double[] x1Plush=new double[taille];

    //x(t+h/2)
    double[] x2aPlushDemi=new double[taille];

    //x(t+h) obtenu a partir de x(t+h/2)
    double[] x2Plush=new double[taille];

    //erreur relative commise lors de l'integration
    double[] delta = new double[taille]; //contiendra |x1Plush-x2Plush|/(|x1Plush|+|x2Plush|+x') i.e l'erreur relative commise lors de l'integration 


    //initialisation du pas maximum h
    double hmax=intTps;


    //mise a jour de la variable GLOBALE tFinal
    //tps a l'interpolation ou l'on demande la solution
    tFinal=tFinal+intTps;



    //si le pas h est nul, faut bien lui donner qq chose
    //En fait, lorsqu'on appel la fonction la premiere fois, le pas h est 
    //egale a 0. Il faut donc l'initialiser a qq chose.
    //On donne l'interval de temps d'integration (hmax), donc, on initialise 
    //ce pas h avec ce intTps.


    if(h==0){
      h=hmax;
    }

    //sinon, on ne tient pas compte de intTps pour determiner h, on utilise le 
    //pas h qui est declare en global car on a deja appele la methode 
    //precedemment.


    if(! vieuxCalcul){ //c'est un nouveau calcul

      //initialisation de xNew[],variable utilisee ds ts les calculs suivants
      for(int i=0;i<taille;i++){
        xNew[i]=x[i];
      }

      //initialisation de l'erreur pour pouvoir, lors du premier passage 
      //dans la boucle do{...}while(erreur>precision), commencer avec un pas 
      //qui vaut h (en effet, cette boucle commence par une adaptation du 
      //pas !
      erreur=precision*Math.pow(0.9,5);
    }
    //else on est dans la suite d'un calcul deja commence donc, ne tient pas 
    //compte de x donne en parametre et on redemarre avec l'ancien xNew


    //pour pouvoir rentrer dans la boucle qui suit
    boolean dernier=false;
  
    while(! dernier) { //"presque" equivalent a tNew<tFinal)

      //recherche du pas optimal pour calculer le xNew
      do { 
        dernier=false;
 
        //adaptation du pas : - augmente si on entre dans la boucle car 
        //                      erreur<precision. A cause du facteur 
        //                      (de securite) O.9, il y a une augmentation 
        //                      lorsque erreur<1.6935088
        //                    - diminue si on viens de la fin de la boucle car 
        //                      erreur>precision
        h=h*0.9*Math.pow(precision/erreur,0.2);


        //si le pas calcule est plus grand que le pas maximum autorise
        if(h>hmax) {
          h=hmax;
        //---Debugging----
        //System.out.println("-----------h>hmax => h=hmax=" + h);
        //---Debugging----
      }
  
      //si le pas h calcule est tel qu'avec celui-ci, on donnera une 
      //solution a un temps plus grand que celui demande, alors, on adapte le
      //pas en l'egalant au pas qu'il faudrait en partant de tNew pour 
      //pouvoir finir au temps exacte demande (tFinal).
      if(h> tFinal-tNew ) {
        h=tFinal-tNew;

        dernier=true;//puisqu'on adapte le pas de cette facon, c'est 
                     //qu'apres, il n'y aura plus d'autre pas.
      }

      //calcul de x(t+h)
      x1Plush=rk4(xNew,h,aIntegrer);

      //calcul de x(t+h/2)
      x2aPlushDemi=rk4(xNew,h/2,aIntegrer);

      //calcul de x(t+h/2+h/2)
      x2Plush=rk4(x2aPlushDemi,h/2,aIntegrer);


      //pour que l'erreur ne devienne pas infinie lors du passage d'un cote
      // a l'autre de l'axe des x, on ajoutera yPrime*h au denominateur
      double[] yPrime = new double[taille];
      yPrime = aIntegrer.fonction_multi(x2Plush,parametres,taille);


      //calcul de l'erreur commise
      for(int i=0;i<taille;i++){
        delta[i]=Math.abs( x2Plush[i]-x1Plush[i] ) / ( Math.abs(x2Plush[i]) + Math.abs(x1Plush[i]) + Math.abs(yPrime[i]*h));
      }

      erreur=Math.abs(maximum(delta));



    } while(erreur>precision);

    //arriver ici, on a un pas optimal h et on met enfin a jour notre xNew et
    //tNew

    //actualiastion de xNew
    for(int i=0;i<taille;i++){
      xNew[i]=x2Plush[i] + (x2Plush[i]-x1Plush[i])/15;
      //xNew a alors une erreur d'ordre 6 au lieu de l'ordre 5
    }

    //acutalisation de tNew
    tNew=tNew+h;
  

    }//fin de while(tNew<tFinal)


    double[] xResultatT=new double[taille];

    for(int i=0; i<taille ; i++) {
      xResultatT[i]=xNew[i];
    }

    return xResultatT;
  }



  /**
   * Appelle la fonction rk4pv(double[] x, double deltaT, double precision, boolean vieuxCalcul) sachant que on est dans la suite d'un vieux calcul. Donc, on passe un x bidon comme premier parametre et "true" pour le parametre boolean
   * Voir rk4pv(double[] x,double deltaT, double precision, boolean vieuxCalcul)
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double deltaT, double precision, ChoixDeFonction aIntegrer){
    double[] xResultatT=new double[taille]; //tableau de resultat au temps deltaT exactement !!!
    double[] xBidon=new double[taille];
    for(int i=0;i<taille;i++){
      xBidon[i]=0;
    }
    xResultatT=rk4pv(xBidon,deltaT,precision,aIntegrer, true);
    return xResultatT;
  }



  /**
   * Appelle la fonction rk4pv(double[] x, double deltaT, double precision, boolean vieuxCalcul) avec "false" pour le parametre boolean
   * Voir rk4pv(double[] x,double deltaT, double precision, boolean vieuxCalcul)
   * @since Projet Trois Corps 2006
   */

  public double[] rk4pv(double[] x,double deltaT, double precision,ChoixDeFonction aIntegrer){
    double[] xResultatT=new double[taille]; //tableau de resultat au temps deltaT exactement !!!
    xResultatT=rk4pv(x,deltaT,precision,aIntegrer, false);
    return xResultatT;
  }



}//fin de public class
